@@ -69,7 +69,7 @@ typedef enum {
 typedef struct pipeinfo_T {
     int pi_fromprevfd;   /* reading end of the pipe from the previous process */
     int pi_tonextfds[2]; /* both ends of the pipe to the next process */
-    /* for each member, -1 is assigned if a pipe is unused */
+    /* -1 is assigned to unused members. */
 } pipeinfo_T;
 #define PIPEINFO_INIT { -1, { -1, -1 }, }
 
@@ -161,16 +161,16 @@ static int exec_iteration(void *const *commands, const char *codename)
 /* exit status of the last command */
 int laststatus = Exit_SUCCESS;
 /* exit status of the command preceding the currently executed trap action */
-int savelaststatus = -1;  // -1 if not in a trap action
+int savelaststatus = -1;  // -1 if not in a trap handler
 /* exit status of the last command substitution */
 static int lastcmdsubstatus;
 /* the process ID of the last asynchronous list */
 pid_t lastasyncpid;
 
 /* This flag is set to true while the shell is executing the condition of an if-
- * statement, an and-or list, etc. to suppress the effect of "-o errexit"
+ * statement, an and-or list, etc. to suppress the effect of the "errexit"
  * option. */
-static bool supresserrexit;
+static bool supresserrexit = false;
 
 /* state of currently executed loop */
 static struct execinfo {
@@ -190,6 +190,9 @@ static struct iterinfo {
     enum { ie_none, ie_continue, ie_break } exception;
 } iterinfo;
 
+/* This flag is set when a special builtin is executed as such. */
+bool special_builtin_executed;
+
 /* This flag is set while the "exec" builtin is executed. */
 static bool exec_builtin_executed = false;
 
@@ -220,7 +223,7 @@ struct execinfo *save_execinfo(void)
     return save;
 }
 
-/* Restores `execinfo' and frees the saved info. */
+/* Restores `execinfo' to `save' and frees `save'. */
 void load_execinfo(struct execinfo *save)
 {
     execinfo = *save;
@@ -240,7 +243,7 @@ bool need_break(void)
 }
 
 
-/* Executes and-or lists.
+/* Executes the and-or lists.
  * If `finally_exit' is true, the shell exits after execution. */
 void exec_and_or_lists(const and_or_T *a, bool finally_exit)
 {
@@ -256,7 +259,7 @@ void exec_and_or_lists(const and_or_T *a, bool finally_exit)
 	exit_shell();
 }
 
-/* Executes pipelines. */
+/* Executes the pipelines. */
 void exec_pipelines(const pipeline_T *p, bool finally_exit)
 {
     for (bool first = true; p && !need_break(); p = p->next, first = false) {
@@ -278,7 +281,7 @@ void exec_pipelines(const pipeline_T *p, bool finally_exit)
 	exit_shell();
 }
 
-/* Executes pipelines asynchronously. */
+/* Executes the pipelines asynchronously. */
 void exec_pipelines_async(const pipeline_T *p)
 {
     if (!p->next && !p->pl_neg) {
@@ -303,7 +306,7 @@ void exec_pipelines_async(const pipeline_T *p)
 	    job->j_pcount = 1;
 
 	    set_active_job(job);
-	    add_job(false);
+	    add_job(shopt_curasync);
 	    laststatus = Exit_SUCCESS;
 	    lastasyncpid = cpid;
 	} else if (cpid == 0) {
@@ -318,7 +321,7 @@ void exec_pipelines_async(const pipeline_T *p)
     }
 }
 
-/* Executes an if command */
+/* Executes the if command */
 void exec_if(const command_T *c, bool finally_exit)
 {
     assert(c->c_type == CT_IF);
@@ -352,7 +355,7 @@ bool exec_condition(const and_or_T *c)
     }
 }
 
-/* Executes a for command. */
+/* Executes the for command. */
 void exec_for(const command_T *c, bool finally_exit)
 {
     assert(c->c_type == CT_FOR);
@@ -362,11 +365,13 @@ void exec_for(const command_T *c, bool finally_exit)
     void **words;
 
     if (c->c_forwords) {
+	/* expand the words between "in" and "do" of the for command. */
 	if (!expand_line(c->c_forwords, &count, &words)) {
 	    laststatus = Exit_EXPERROR;
 	    goto finish;
 	}
     } else {
+	/* no "in" keyword in the for command: use the positional parameters */
 	struct get_variable v = get_variable(L"@");
 	assert(v.type == GV_ARRAY && v.values != NULL);
 	words = duparray(v.values, copyaswcs);
@@ -399,7 +404,7 @@ void exec_for(const command_T *c, bool finally_exit)
     }
 
 done:
-    while (++i < count)
+    while (++i < count)  /* free unused words */
 	free(words[i]);
     free(words);
     if (count == 0 && c->c_forcmds)
@@ -410,8 +415,8 @@ finish:
 	exit_shell();
 }
 
-/* Executes a while/until command. */
-/* The exit status of the while/until command is that of `c_whlcmds' executed
+/* Executes the while/until command. */
+/* The exit status of a while/until command is that of `c_whlcmds' executed
  * last.  If `c_whlcmds' is not executed at all, the status is 0 regardless of
  * `c_whlcond'. */
 void exec_while(const command_T *c, bool finally_exit)
@@ -429,8 +434,10 @@ void exec_while(const command_T *c, bool finally_exit)
 
 	if (cond != c->c_whltype)
 	    break;
-	exec_and_or_lists(c->c_whlcmds, false);
-	status = laststatus;
+	if (c->c_whlcmds) {
+	    exec_and_or_lists(c->c_whlcmds, false);
+	    status = laststatus;
+	}
 
 	if (!c->c_whlcmds)
 	    handle_signals();
@@ -445,7 +452,7 @@ done:
 }
 #undef CHECK_LOOP
 
-/* Executes a case command. */
+/* Executes the case command. */
 void exec_case(const command_T *c, bool finally_exit)
 {
     assert(c->c_type == CT_CASE);
@@ -492,10 +499,10 @@ fail:
     goto done;
 }
 
-/* Updates the contents of a `pipeinfo_T' to proceed to the next process
+/* Updates the contents of the `pipeinfo_T' to proceed to the next process
  * execution. `pi->pi_fromprevfd' and `pi->pi_tonextfds[PIDX_OUT]' are closed,
  * `pi->pi_tonextfds[PIDX_IN]' is moved to `pi->pi_fromprevfd', and,
- * if `next' is true, a new pipe is opened in `pi->pi_tonextfds' or
+ * if `next' is true, a new pipe is opened in `pi->pi_tonextfds' or,
  * if `next' is false, `pi->pi_tonextfds' is assigned -1.
  * Returns true iff successful. */
 void next_pipe(pipeinfo_T *pi, bool next)
@@ -506,21 +513,31 @@ void next_pipe(pipeinfo_T *pi, bool next)
 	xclose(pi->pi_tonextfds[PIDX_OUT]);
     pi->pi_fromprevfd = pi->pi_tonextfds[PIDX_IN];
     if (next) {
-	/* If the file descriptors 0 and/or 1 are unused, we open a dummy pipe
-	 * so that pipes really used have the file descriptors larger than 1.
-	 * If we don't do this, the pipe's file descriptors are overridden when
-	 * we connect the pipe to stdin/stdout later. */
-	int dummy[2];
-	bool usedummy = (fcntl(STDIN_FILENO,  F_GETFD) == -1 && errno == EBADF)
-		     || (fcntl(STDOUT_FILENO, F_GETFD) == -1 && errno == EBADF);
-
-	if (usedummy && pipe(dummy) < 0)
-	    goto fail;
 	if (pipe(pi->pi_tonextfds) < 0)
 	    goto fail;
-	if (usedummy) {
-	    xclose(dummy[PIDX_IN]);
-	    xclose(dummy[PIDX_OUT]);
+
+	/* The pipe's FDs must not be 0 or 1, or they may be overridden by each
+	 * other when we connect the pipe later. If they are 0 or 1, move them
+	 * to bigger numbers. */
+	int origin  = pi->pi_tonextfds[PIDX_IN];
+	int origout = pi->pi_tonextfds[PIDX_OUT];
+	if (origin < 2 || origout < 2) {
+	    if (origin < 2)
+		pi->pi_tonextfds[PIDX_IN] = dup(origin);
+	    if (origout < 2)
+		pi->pi_tonextfds[PIDX_OUT] = dup(origout);
+	    if (origin < 2)
+		xclose(origin);
+	    if (origout < 2)
+		xclose(origout);
+	    if (pi->pi_tonextfds[PIDX_IN] < 0) {
+		xclose(pi->pi_tonextfds[PIDX_OUT]);
+		goto fail;
+	    }
+	    if (pi->pi_tonextfds[PIDX_OUT] < 0) {
+		xclose(pi->pi_tonextfds[PIDX_IN]);
+		goto fail;
+	    }
 	}
     } else {
 	pi->pi_tonextfds[PIDX_IN] = pi->pi_tonextfds[PIDX_OUT] = -1;
@@ -547,7 +564,7 @@ void connect_pipes(pipeinfo_T *pi)
 	xclose(pi->pi_tonextfds[PIDX_IN]);
 }
 
-/* Executes commands in a pipeline. */
+/* Executes the commands in a pipeline. */
 void exec_commands(command_T *c, exec_T type)
 {
     size_t count;
@@ -558,7 +575,7 @@ void exec_commands(command_T *c, exec_T type)
     pipeinfo_T pinfo = PIPEINFO_INIT;
     commandtype_T lasttype;
 
-    /* count the number of commands */
+    /* count the number of the commands */
     count = 0;
     for (cc = c; cc; cc = cc->next)
 	count++;
@@ -567,7 +584,7 @@ void exec_commands(command_T *c, exec_T type)
     job = xmalloc(sizeof *job + count * sizeof *job->j_procs);
     ps = job->j_procs;
 
-    /* execute commands */
+    /* execute the commands */
     pgid = 0, cc = c, pp = ps;
     do {
 	pid_t pid;
@@ -593,13 +610,12 @@ void exec_commands(command_T *c, exec_T type)
     } while (cc != NULL);
     assert(type != execself); /* `exec_process' doesn't return for `execself' */
     assert(pinfo.pi_tonextfds[PIDX_IN] < 0);
+    assert(pinfo.pi_tonextfds[PIDX_OUT] < 0);
     if (pinfo.pi_fromprevfd >= 0)
 	xclose(pinfo.pi_fromprevfd);           /* close leftover pipes */
-    if (pinfo.pi_tonextfds[PIDX_OUT] >= 0)
-	xclose(pinfo.pi_tonextfds[PIDX_OUT]);  /* close leftover pipes */
 
     if (pgid == 0) {
-	/* no more things to do if didn't fork */
+	/* nothing more to do if we didn't fork */
 	free(job);
     } else {
 	job->j_pgid = doing_job_control_now ? pgid : 0;
@@ -620,7 +636,7 @@ void exec_commands(command_T *c, exec_T type)
 	}
 	if (job->j_status != JS_DONE) {
 	    for (cc = c, pp = ps; cc != NULL; cc = cc->next, pp++)
-		pp->pr_name = command_to_wcs(cc);
+		pp->pr_name = command_to_wcs(cc, false);
 	    add_job(type == execnormal || shopt_curasync);
 	} else {
 	    notify_signaled_job(ACTIVE_JOBNO);
@@ -631,10 +647,10 @@ void exec_commands(command_T *c, exec_T type)
     handle_signals();
     if (shopt_errexit && !supresserrexit && laststatus != Exit_SUCCESS
 	    && lasttype == CT_SIMPLE)
-	exit_shell();
+	exit_shell_with_status(laststatus);
 }
 
-/* Executes a command.
+/* Executes the command.
  * If job control is active, the child process's process group ID is set to
  * `pgid'. If `pgid' is 0, set to the child process's process ID.
  * If a child process forked successfully, its process ID is returned.
@@ -672,6 +688,10 @@ pid_t exec_process(
 	    laststatus = Exit_EXPERROR;
 	    goto done;
 	}
+	if (is_interrupted()) {
+	    recfree(argv, free);
+	    goto done;
+	}
 	if (argc == 0) {
 	    argv0 = NULL;
 	} else {
@@ -698,7 +718,6 @@ pid_t exec_process(
 	goto done2;
     }
     
-    /* redirect stdin to "/dev/null" if non-interactive and asynchronous */
     if (type == execasync && pi->pi_fromprevfd < 0)
 	maybe_redirect_stdin_to_devnull();
 
@@ -723,13 +742,13 @@ pid_t exec_process(
 
     /* First, we check if the command is a special builtin or a function
      * and determine whether we have to open a temporary environment. */
-    search_command(argv0, argv[0], &cmdinfo,
-	    sct_builtin | sct_function);
+    search_command(argv0, argv[0], &cmdinfo, sct_builtin | sct_function);
+    special_builtin_executed = (cmdinfo.type == specialbuiltin);
     temp = c->c_assigns && assignment_is_temporary(cmdinfo.type);
     if (temp)
 	open_new_environment(true);
 
-    /* perform assignments */
+    /* perform the assignments */
     if (!do_assignments(c->c_assigns, temp, true)) {
 	/* On assignment error, the command is not executed. */
 	print_xtrace(NULL);
@@ -758,7 +777,7 @@ pid_t exec_process(
 		    if (command_not_found_handler(argv))
 			goto done3;
 	    }
-	    /* If the command was not found and the shell is not in POSIXly
+	    /* If the command was not found and the shell is not in the POSIXly
 	     * correct mode, the not-found handler is called. */
 	}
     }
@@ -807,17 +826,19 @@ done:
  * changed.
  * If job control is active and `fg' is true, the child process becomes a
  * foreground process.
- * `sigtype' specifies settings of signals in the child.
+ * `sigtype' specifies the settings of signals in the child.
  * `sigtype' is a bitwise OR of the followings:
  *   t_quitint: SIGQUIT & SIGINT are ignored if the parent's job control is off
  *   t_tstp: SIGTSTP is ignored if the parent is job-controlling
- *   t_leave: don't clear traps and shellfds. This option should be used only if
- *          the shell is going to `exec' to an extenal program.
+ *   t_leave: Don't clear traps and shellfds. Restore the signal mask for
+ *          SIGCHLD. This option must be used iff the shell is going to `exec'
+ *          to an extenal program.
  * Returns the return value of `fork'. */
 pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 {
     sigset_t savemask;
     if (sigtype & (t_quitint | t_tstp)) {
+	/* block all signals to prevent the race condition */
 	sigset_t all;
 	sigfillset(&all);
 	sigemptyset(&savemask);
@@ -867,8 +888,8 @@ pid_t fork_and_reset(pid_t pgid, bool fg, sigtype_T sigtype)
 
 /* Searches and determines the command to execute.
  * The result is assigned to `*ci'.
- * If the command is not found, `ci->type' is `externalprogram' and
- * `ci->ci_path' is NULL.
+ * If the command is not found, `ci->type' is set to `externalprogram' and
+ * `ci->ci_path' is set to NULL.
  * `name' and `wname' must contain the same string value. */
 /* When the result type `ci->type' is `regularbuiltin', the result value is
  * either a function pointer to the builtin (if the `sct_rbpath' flag is not
@@ -918,7 +939,7 @@ void search_command(
 
     ci->ci_path = (type & sct_defpath)
 	? get_command_path_default(name) : get_command_path(name, false);
-    if (bi && (ci->ci_path || (!posixly_correct && (type & sct_rbpath)))) {
+    if (bi && (ci->ci_path || !posixly_correct)) {
 	assert(bi->type == BI_REGULAR);
 	ci->type = regularbuiltin;
 	if (!(type & sct_rbpath))
@@ -960,8 +981,9 @@ bool assignment_is_temporary(enum cmdtype_T type)
 }
 
 /* Executes $COMMAND_NOT_FOUND_HANDLER if any.
- * Returns true if the hander is executed and $HANDLED is set non-empty.
- * Returns false otherwise. */
+ * `argv' is set to the positional parameters of the environment in which the
+ * handler is executed.
+ * Returns true iff the hander was executed and $HANDLED was set non-empty. */
 bool command_not_found_handler(void *const *argv)
 {
     static bool handling = false;
@@ -996,13 +1018,16 @@ bool command_not_found_handler(void *const *argv)
     }
 }
 
-/* Executes a command whose type is not `CT_SIMPLE'.
+/* Executes the specified command whose type is not `CT_SIMPLE'.
  * The redirections for the command is not performed in this function.
  * For CT_SUBSHELL, this function must be called in an already-forked subshell.
  */
 void exec_nonsimple_command(command_T *c, bool finally_exit)
 {
+    /* increment the reference count of `c' to prevent `c' from being freed
+     * during execution. */
     c = comsdup(c);
+
     switch (c->c_type) {
     case CT_SIMPLE:
 	assert(false);
@@ -1031,11 +1056,12 @@ void exec_nonsimple_command(command_T *c, bool finally_exit)
 	    exit_shell();
 	break;
     }
+
     comsfree(c);
 }
 
-/* Executes a simple command. */
-/* `argv0' is a multibyte version of `argv[0]' */
+/* Executes the simple command. */
+/* `argv0' is the multibyte version of `argv[0]' */
 void exec_simple_command(
 	const commandinfo_T *ci, int argc, char *argv0, void **argv,
 	bool finally_exit)
@@ -1104,7 +1130,7 @@ xwcsbuf_T *get_xtrace_buffer(void)
     return &xtrace_buffer;
 }
 
-/* Prints a trace if "-o xtrace" option is on. */
+/* Prints a trace if the "xtrace" option is on. */
 void print_xtrace(void *const *argv)
 {
     bool tracevars = xtrace_buffer.contents != NULL
@@ -1134,7 +1160,7 @@ void print_xtrace(void *const *argv)
     }
 }
 
-/* Executes a command script by `exec'ing a shell.
+/* Executes the specified command as a shell script by `exec'ing a shell.
  * `path' is the full path to the script file.
  * Returns iff failed to `exec'. */
 void exec_fall_back_on_sh(
@@ -1156,9 +1182,16 @@ void exec_fall_back_on_sh(
 	args[index++] = (char *) path;
     for (int i = 1; i < argc; i++)
 	args[index++] = argv[i];
-    args[index++] = NULL;
+    args[index] = NULL;
 #if HAVE_PROC_SELF_EXE
     xexecve("/proc/self/exe", args, envp);
+#elif HAVE_PROC_CURPROC_FILE
+    xexecve("/proc/curproc/file", args, envp);
+#elif HAVE_PROC_OBJECT_AOUT
+    char *objpath = malloc_printf(
+	    "/proc/%jd/object/a.out", (intmax_t) getpid());
+    xexecve(objpath, args, envp);
+    free(objpath);
 #elif HAVE_PATHS_H && defined _PATH_BSHELL
     xexecve(_PATH_BSHELL, args, envp);
 #else
@@ -1209,16 +1242,18 @@ int xexecve(const char *path, char *const *argv, char *const *envp)
     return -1;
 }
 
-/* Executes a command substitution and returns the string to substitute with.
- * This function block until the command finishes.
- * The returned string is newly malloced and has the trailing newlines removed.
+/* Executes the command substitution and returns the string to substitute with.
+ * This function blocks until the command finishes.
+ * The return value is a newly-malloced string without a trailing newline.
  * NULL is returned on error. */
-wchar_t *exec_command_substitution(const wchar_t *code)
+wchar_t *exec_command_substitution(const embedcmd_T *cmdsub)
 {
     int pipefd[2];
     pid_t cpid;
 
-    if (!code[0])
+    if (cmdsub->is_preparsed
+	    ? cmdsub->value.preparsed == NULL
+	    : cmdsub->value.unparsed[0] == L'\0')  /* command is empty? */
 	return xwcsdup(L"");
 
     /* open a pipe to receive output from the command */
@@ -1228,8 +1263,8 @@ wchar_t *exec_command_substitution(const wchar_t *code)
     }
 
     /* If the child is stopped by SIGTSTP, it can never be resumed and
-     * the shell will be stuck. So we make the child unstoppable
-     * by SIGTSTP. */
+     * the shell will be stuck. So we make the child unstoppable by SIGTSTP.
+     * The `t_tstp' flag designates this behavior. */
     cpid = fork_and_reset(-1, false, t_tstp);
     if (cpid < 0) {
 	/* fork failure */
@@ -1271,9 +1306,10 @@ wchar_t *exec_command_substitution(const wchar_t *code)
 	laststatus = savelaststatus;
 
 	/* trim trailing newlines and return */
-	while (buf.length > 0 && buf.contents[buf.length - 1] == L'\n')
-	    wb_remove(&buf, buf.length - 1, 1);
-	return wb_towcs(&buf);
+	size_t len = buf.length;
+	while (len > 0 && buf.contents[len - 1] == L'\n')
+	    len--;
+	return wb_towcs(wb_truncate(&buf, len));
     } else {
 	/* child process */
 	xclose(pipefd[PIDX_IN]);
@@ -1283,7 +1319,10 @@ wchar_t *exec_command_substitution(const wchar_t *code)
 	    xclose(pipefd[PIDX_OUT]);
 	}
 
-	exec_wcs(code, gt("command substitution"), true);
+	if (cmdsub->is_preparsed)
+	    exec_and_or_lists(cmdsub->value.preparsed, true);
+	else
+	    exec_wcs(cmdsub->value.unparsed, gt("command substitution"), true);
 	assert(false);
     }
 }
@@ -1293,7 +1332,6 @@ wchar_t *exec_command_substitution(const wchar_t *code)
  * If the iteration is interrupted by the "break -i" command, the remaining
  * elements are not executed.
  * `codename' is passed to `exec_wcs' as the command name.
- * If `saveparsestate' is true, the parser's state is saved during execution.
  * Returns the exit status of the executed command (or zero if none executed).
  * When this function returns, `laststatus' is restored to the original value.*/
 int exec_iteration(void *const *commands, const char *codename)
@@ -1315,12 +1353,12 @@ int exec_iteration(void *const *commands, const char *codename)
 }
 
 /* Executes the value of the specified variable.
- * The variable value is parsed as a command string.
+ * The variable value is parsed as commands.
  * If the `varname' names an array, every element of the array is executed (but
  * if the iteration is interrupted by the "break -i" command, the remaining
  * elements are not executed).
  * `codename' is passed to `exec_wcs' as the command name.
- * Returns the exit status of the executed command (or zero if none executed or
+ * Returns the exit status of the executed command (or zero if none executed, or
  * -1 if the variable is unset).
  * When this function returns, `laststatus' is restored to the original value.*/
 int exec_variable_as_commands(const wchar_t *varname, const char *codename)
@@ -1339,7 +1377,7 @@ int exec_variable_as_commands(const wchar_t *varname, const char *codename)
 	    array = duparrayn(gv.values, gv.count, copyaswcs);
 	    break;
 	case GV_ARRAY_CONCAT:
-	    /* should execute the concatenated value, but not supported now */
+	    /* should execute the concatenated value, but is not supported now*/
 	    return -1;
 	default:
 	    assert(false);
@@ -1368,11 +1406,13 @@ static void print_command_absolute_path(
 /* Options for the "break", "continue" and "eval" builtins. */
 static const struct xoption iter_options[] = {
     { L"iteration", xno_argument, L'i', },
+#if YASH_ENABLE_HELP
     { L"help",      xno_argument, L'-', },
+#endif
     { NULL, 0, 0, },
 };
 
-/* "return" builtin */
+/* The "return" builtin */
 int return_builtin(int argc, void **argv)
 {
     wchar_t opt;
@@ -1380,8 +1420,10 @@ int return_builtin(int argc, void **argv)
     xoptind = 0, xopterr = true;
     while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
 	switch (opt) {
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr, gt("Usage:  %ls [n]\n"), ARGV(0));
 		SPECIAL_BI_ERROR;
@@ -1394,9 +1436,7 @@ int return_builtin(int argc, void **argv)
     int status;
     const wchar_t *statusstr = ARGV(xoptind);
     if (statusstr != NULL) {
-	if (xwcstoi(statusstr, 10, &status) && status >= 0) {
-	    status &= 0xFF;
-	} else {
+	if (!xwcstoi(statusstr, 10, &status) || status < 0) {
 	    xerror(0, Ngt("`%ls' is not a valid integer"), statusstr);
 	    status = Exit_ERROR;
 	    SPECIAL_BI_ERROR;
@@ -1409,6 +1449,7 @@ int return_builtin(int argc, void **argv)
     return status;
 }
 
+#if YASH_ENABLE_HELP
 const char return_help[] = Ngt(
 "return - return from function\n"
 "\treturn [n]\n"
@@ -1416,8 +1457,10 @@ const char return_help[] = Ngt(
 "of <n>. If <n> is not specified, it defaults to the exit status of the last\n"
 "executed command. <n> should be between 0 and 255 inclusive.\n"
 );
+#endif
 
-/* "break" and "continue" builtin */
+/* The "break"/"continue" builtin, which accepts the following option:
+ *  -i: iterative execution */
 int break_builtin(int argc, void **argv)
 {
     wchar_t opt;
@@ -1429,8 +1472,10 @@ int break_builtin(int argc, void **argv)
 	    case L'i':
 		iter = true;
 		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr,
 			gt("Usage:  %ls [n]\n        %ls -i\n"),
@@ -1443,6 +1488,7 @@ int break_builtin(int argc, void **argv)
 	goto print_usage;
 
     if (iter) {
+	/* break/continue iteration */
 	if (!iterinfo.iterating) {
 	    xerror(0, Ngt("not in iteration"));
 	    return Exit_ERROR;
@@ -1495,6 +1541,8 @@ int break_builtin(int argc, void **argv)
     }
 }
 
+#if YASH_ENABLE_HELP
+
 const char break_help[] = Ngt(
 "break - exit loop\n"
 "\tbreak [n]\n"
@@ -1516,21 +1564,29 @@ const char continue_help[] = Ngt(
 "commands and resumes the next iteration of the current iterative execution.\n"
 );
 
-/* The "eval" builtin */
+#endif /* YASH_ENABLE_HELP */
+
+/* The "eval" builtin, which accepts the following option:
+ *  -i: iterative execution */
 int eval_builtin(int argc __attribute__((unused)), void **argv)
 {
     bool iter = false;
     wchar_t opt;
     xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"+i", iter_options, NULL))) {
+    while ((opt = xgetopt_long(argv,
+		    posixly_correct ? L"+" : L"+i", iter_options, NULL))) {
 	switch (opt) {
 	    case L'i':
 		iter = true;
 		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
-		fprintf(stderr, gt("Usage:  eval [-i] [arg...]\n"));
+		fprintf(stderr, gt(posixly_correct
+			    ? Ngt("Usage:  eval [arg...]\n")
+			    : Ngt("Usage:  eval [-i] [arg...]\n")));
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	}
@@ -1546,6 +1602,7 @@ int eval_builtin(int argc __attribute__((unused)), void **argv)
     }
 }
 
+#if YASH_ENABLE_HELP
 const char eval_help[] = Ngt(
 "eval - evaluate arguments as command\n"
 "\teval [-i] [arg...]\n"
@@ -1554,18 +1611,41 @@ const char eval_help[] = Ngt(
 "with a space inserted between each <arg> and the whole resultant string is\n"
 "parsed at a time. With the -i option, <arg>s are parsed and executed one by\n"
 "one (iterative execution).\n"
+"In POSIXly correct mode, the -i option cannot be used.\n"
 );
+#endif
 
+/* The "." builtin, which accepts the following option:
+ *  -A: disable aliases */
 int dot_builtin(int argc, void **argv)
 {
+    static const struct xoption long_options[] = {
+	{ L"no-alias", xno_argument, L'A', },
+#if YASH_ENABLE_HELP
+	{ L"help",     xno_argument, L'-', },
+#endif
+	{ NULL, 0, 0, },
+    };
+
+    bool enable_alias = true;
+
     wchar_t opt;
     xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
+    while ((opt = xgetopt_long(argv,
+		    posixly_correct ? L"+" : L"+A",
+		    long_options, NULL))) {
 	switch (opt) {
+	    case L'A':
+		enable_alias = false;
+		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
-		fprintf(stderr, gt("Usage:  . file [arg...]\n"));
+		fprintf(stderr, gt(posixly_correct
+			    ? Ngt("Usage:  . file\n")
+			    : Ngt("Usage:  . [-A] file [arg...]\n")));
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	}
@@ -1575,6 +1655,10 @@ int dot_builtin(int argc, void **argv)
     if (!filename)
 	goto print_usage;
 
+    bool has_args = xoptind < argc;
+    if (has_args && posixly_correct)
+	goto print_usage;
+
     char *mbsfilename = malloc_wcstombs(filename);
     if (!mbsfilename) {
 	xerror(EILSEQ, Ngt("unexpected error"));
@@ -1583,8 +1667,7 @@ int dot_builtin(int argc, void **argv)
 
     char *path;
     if (!wcschr(filename, L'/')) {
-	path = which(mbsfilename, get_path_array(PA_PATH),
-		is_readable_regular);
+	path = which(mbsfilename, get_path_array(PA_PATH), is_readable_regular);
 	if (!path) {
 	    if (!posixly_correct) {
 		path = mbsfilename;
@@ -1600,8 +1683,7 @@ int dot_builtin(int argc, void **argv)
 	path = mbsfilename;
     }
 
-    bool exp = xoptind < argc;
-    if (exp) {
+    if (has_args) {
 	open_new_environment(false);
 	set_positional_parameters(argv + xoptind);
     }
@@ -1618,39 +1700,47 @@ int dot_builtin(int argc, void **argv)
 	return Exit_FAILURE;
     }
 
-    exec_input(f, mbsfilename, false, false);
+    exec_input(f, mbsfilename, false, enable_alias, false);
     fclose(f);
     free(mbsfilename);
 
-    if (exp) {
+    if (has_args) {
 	close_current_environment();
     }
 
     return laststatus;
 }
 
+#if YASH_ENABLE_HELP
 const char dot_help[] = Ngt(
 "dot - read file and execute commands\n"
-"\t. file [arg...]\n"
+"\t. [-A] file [arg...]\n"
 "Reads the specified <file> and executes commands in it.\n"
 "If <arg>s are specified, they are used as the positional parameters.\n"
 "Otherwise, the positional parameters are not changed.\n"
 "If <file> does not contain any slashes, the shell searches $PATH for a\n"
 "readable shell script file whose name is <file>. To ensure that the file in\n"
 "the current working directory is used, start <file> with \"./\".\n"
+"If the -A (--no-alias) option is specified, alias substitution is not\n"
+"performed during processing the file.\n"
+"In POSIXly correct mode, the -A option cannot be used and <arg>s must not be\n"
+"given.\n"
 );
+#endif
 
 /* The "exec" builtin, which accepts the following options:
- * -a name: give <name> as argv[0] to the command
- * -c: don't pass environment variables to the command
- * -f: suppress error when we have stopped jobs */
+ *  -a name: give <name> as argv[0] to the command
+ *  -c: don't pass environment variables to the command
+ *  -f: suppress error when we have stopped jobs */
 int exec_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
 	{ L"as",    xrequired_argument, L'a', },
 	{ L"clear", xno_argument,       L'c', },
 	{ L"force", xno_argument,       L'f', },
+#if YASH_ENABLE_HELP
 	{ L"help",  xno_argument,       L'-', },
+#endif
 	{ NULL, 0, 0, },
     };
 
@@ -1660,17 +1750,20 @@ int exec_builtin(int argc, void **argv)
 
     xoptind = 0, xopterr = true;
     while ((opt = xgetopt_long(argv,
-		    posixly_correct ? L"" : L"+a:cf",
+		    posixly_correct ? L"+" : L"+a:cf",
 		    long_options, NULL))) {
 	switch (opt) {
 	    case L'a':  as = xoptarg;  break;
 	    case L'c':  clear = true;  break;
 	    case L'f':  force = true;  break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
-		fprintf(stderr,
-		    gt("Usage:  exec [-cf] [-a name] [command [arg...]]\n"));
+		fprintf(stderr, gt(posixly_correct
+		    ? Ngt("Usage:  exec [command [arg...]]\n")
+		    : Ngt("Usage:  exec [-cf] [-a name] [command [arg...]]\n")));
 		SPECIAL_BI_ERROR;
 		return Exit_ERROR;
 	}
@@ -1700,7 +1793,7 @@ int exec_builtin(int argc, void **argv)
 /* Implements the "exec" builtin.
  * argc, argv: the operands (not arguments) of the "exec" builtin.
  * as: value of the -a option or NULL
- * clear: true iff the -c option */
+ * clear: true iff the -c option is specified */
 int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 {
     int err;
@@ -1730,6 +1823,8 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 
     char **envs;
     if (clear) {
+	/* use the environment that contains only the variables assigned by the
+	 * assignment for this `exec' builtin. */
 	plist_T list;
 	
 	pl_init(&list);
@@ -1740,7 +1835,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 	}
 	envs = (char **) pl_toary(&list);
     } else {
-	envs = NULL;
+	envs = environ;
     }
 
     if (as) {
@@ -1754,7 +1849,7 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
     }
 
     restore_signals(true);
-    xexecve(commandpath, args, envs ? envs : environ);
+    xexecve(commandpath, args, envs);
     if (errno != ENOEXEC) {
 	if (errno == EACCES && is_directory(commandpath))
 	    errno = EISDIR;
@@ -1764,29 +1859,31 @@ int exec_builtin_2(int argc, void **argv, const wchar_t *as, bool clear)
 	    xerror(errno, Ngt("cannot execute `%s' (%s)"),
 		    args[0], commandpath);
     } else {
-	exec_fall_back_on_sh(argc, args, envs ? envs : environ, commandpath);
+	exec_fall_back_on_sh(argc, args, envs, commandpath);
     }
     if (posixly_correct || !is_interactive_now)
 	exit(Exit_NOEXEC);
     set_signals();
     err = Exit_NOEXEC;
 
-    recfree((void **) envs, free);
+    if (envs != environ)
+	recfree((void **) envs, free);
 
 err:
     for (int i = 0; i < argc; i++)
 	free(args[i]);
     free(freelater);
     if (posixly_correct || !is_interactive_now)
-	exit_shell_with_status(err);
+	exit(err);
     return err;
 }
 
+#if YASH_ENABLE_HELP
 const char exec_help[] = Ngt(
 "exec - execute command in the shell process\n"
 "\texec [-cf] [-a name] [command [args...]]\n"
-"Replaces the shell process with the specified command. The command is\n"
-"executed in the former shell's process rather than a new child process.\n"
+"Replaces the shell process with the specified command. The shell process is\n"
+"`changed' into the new command's process. No child process is created.\n"
 "When an interactive shell has stopped jobs, the -f (--force) option is\n"
 "required to really do exec.\n"
 "If the -c (--clear) option is specified, the command is executed only with\n"
@@ -1796,9 +1893,10 @@ const char exec_help[] = Ngt(
 "If no <command> is given, the shell does nothing. As a special result,\n"
 "the effects of redirections associated with the \"exec\" command remain\n"
 "after the command.\n"
-"In POSIXly correct mode, none of these options are available and the -f\n"
+"In the POSIXly correct mode, none of these options are available and the -f\n"
 "option is always assumed.\n"
 );
+#endif
 
 /* The "command"/"type" builtin, which accepts the following options:
  *  -b: search builtins
@@ -1814,7 +1912,9 @@ int command_builtin(int argc, void **argv)
 	{ L"standard-path",    xno_argument, L'p', },
 	{ L"identify",         xno_argument, L'v', },
 	{ L"verbose-identify", xno_argument, L'V', },
+#if YASH_ENABLE_HELP
 	{ L"help",             xno_argument, L'-', },
+#endif
 	{ NULL, 0, 0, },
     };
 
@@ -1833,8 +1933,10 @@ int command_builtin(int argc, void **argv)
 	    case L'p':  defpath   = true;  break;
 	    case L'v':  printinfo = true;  humanfriendly = false;  break;
 	    case L'V':  printinfo = true;  humanfriendly = true;   break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
 		goto print_usage;
 	}
@@ -2045,6 +2147,8 @@ void print_command_absolute_path(
     return;
 }
 
+#if YASH_ENABLE_HELP
+
 const char command_help[] = Ngt(
 "command - execute or identify command\n"
 "\tcommand [-bBp] command [argument...]\n"
@@ -2074,6 +2178,8 @@ const char type_help[] = Ngt(
 "Prints the type of <command>s. Same as \"command -V <command>...\".\n"
 );
 
+#endif /* YASH_ENABLE_HELP */
+
 /* The "times" builtin */
 int times_builtin(int argc __attribute__((unused)), void **argv)
 {
@@ -2081,8 +2187,10 @@ int times_builtin(int argc __attribute__((unused)), void **argv)
     xoptind = 0, xopterr = true;
     while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
 	switch (opt) {
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr, gt("Usage:  times\n"));
 		SPECIAL_BI_ERROR;
@@ -2124,6 +2232,7 @@ int times_builtin(int argc __attribute__((unused)), void **argv)
 #undef format_time
 }
 
+#if YASH_ENABLE_HELP
 const char times_help[] = Ngt(
 "times - print process times\n"
 "\ttimes\n"
@@ -2133,6 +2242,7 @@ const char times_help[] = Ngt(
 "processes. For each line, the user time is printed, followed by the system\n"
 "time.\n"
 );
+#endif
 
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
