@@ -65,135 +65,83 @@ static wchar_t *get_prompt(int type)
     __attribute__((malloc,warn_unused_result));
 #endif
 
-/* An input function that inputs from a multibyte string.
- * `inputinfo' is a pointer to a `struct input_mbs_info'.
- * Reads one line from `inputinfo->src', converts it into a wide string, and
- * appends it to the buffer `buf'. `inputinfo->state' must be a valid shift
- * state.
- * If more string is available after reading one line, `inputinfo->src' is
- * updated to point to the character to read next. If no more is available,
- * `inputinfo->src' is assigned NULL and `inputinfo->state' is a initial shift
- * state. */
-int input_mbs(struct xwcsbuf_T *buf, void *inputinfo)
-{
-    struct input_mbs_info *info = inputinfo;
-    size_t initbuflen = buf->length;
-    size_t count;
-
-    if (!info->src)
-	return EOF;
-
-    while (info->srclen > 0) {
-	wb_ensuremax(buf, buf->length + 1);
-	count = mbrtowc(buf->contents + buf->length,
-		info->src, info->srclen, &info->state);
-	switch (count) {
-	    case 0:  /* read a null character */
-		info->src = NULL;
-		info->srclen = 0;
-		return (buf->length == initbuflen) ? EOF : 0;
-	    default:  /* read a non-null character */
-		info->src += count;
-		info->srclen -= count;
-		if (buf->contents[buf->length++] == '\n') {
-		    buf->contents[buf->length] = L'\0';
-		    return 0;
-		}
-		break;
-	    case (size_t) -2:  /* bytes are incomplete */
-	    case (size_t) -1:  /* invalid bytes */
-		goto err;
-	}
-    }
-err:
-    xerror(errno, Ngt("cannot convert multibyte characters "
-		"into wide characters"));
-    return EOF;
-
-    /*
-    wb_ensuremax(buf, buf->length + 120);
-    count = mbsrtowcs(buf->contents + buf->length, &info->src,
-	    buf->maxlength - buf->length + 1, &info->state);
-    if (count == (size_t) -1) {
-	xerror(errno, Ngt("cannot convert multibyte characters "
-		    "into wide characters"));
-	return EOF;
-    }
-    buf->length += count;
-    return 0;
-    */
-}
-
 /* An input function that inputs from a wide string.
  * `inputinfo' is a pointer to a `struct input_wcs_info'.
  * Reads one line from `inputinfo->src' and appends it to the buffer `buf'.
  * If more string is available after reading one line, `inputinfo->src' is
  * updated to point to the character to read next. If no more is available,
  * `inputinfo->src' is assigned NULL. */
-int input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
+inputresult_T input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
 {
     struct input_wcs_info *info = inputinfo;
     const wchar_t *src = info->src;
     size_t count = 0;
 
     if (!src)
-	return EOF;
+	return INPUT_EOF;
 
     while (src[count] != L'\0' && src[count++] != L'\n');
 
     if (count == 0) {
 	info->src = NULL;
-	return EOF;
+	return INPUT_EOF;
     } else {
 	wb_ncat(buf, src, count);
 	info->src = src + count;
-	return 0;
+	return INPUT_OK;
     }
 }
 
 /* An input function that reads input from a file stream.
  * `inputinfo' is a pointer to a `FILE', which must be set to non-blocking.
  * Reads one line with `fgetws' and appends it to the buffer. */
-int input_file(struct xwcsbuf_T *buf, void *inputinfo)
+inputresult_T input_file(struct xwcsbuf_T *buf, void *inputinfo)
 {
     FILE *f = inputinfo;
     int fd = fileno(f);
     size_t initlen = buf->length;
+    bool ok = true;
 
     handle_signals();
-start:
-    wb_ensuremax(buf, buf->length + 100);
-    if (fgetws(buf->contents + buf->length, buf->maxlength - buf->length, f)) {
-	size_t len = wcslen(buf->contents + buf->length);
-	// `len' may be 0 if a null character is input
-	buf->length += len;
-	if (len > 0 && buf->contents[buf->length - 1] != L'\n')
-	    goto start;
-	else
-	    goto end;
-    } else {
-	buf->contents[buf->length] = L'\0';
-	if (feof(f)) {
-	    clearerr(f);
-	    goto end;
-	}
-	assert(ferror(f));
-	switch (errno) {
-	    case EINTR:
-	    case EAGAIN:
-#if EAGAIN != EWOULDBLOCK
-	    case EWOULDBLOCK:
-#endif
-		clearerr(f);
-		wait_for_input(fd, true, -1);
-		goto start;
-	    default:
-		xerror(errno, Ngt("cannot read input"));
+    while (ok) {
+	wb_ensuremax(buf, buf->length + 100);
+	if (fgetws(buf->contents + buf->length,
+		    buf->maxlength - buf->length, f)) {
+	    size_t len = wcslen(buf->contents + buf->length);
+	    // `len' may be 0 if a null character is input
+	    buf->length += len;
+	    if (len == 0 || buf->contents[buf->length - 1] == L'\n')
 		goto end;
+	} else {
+	    buf->contents[buf->length] = L'\0';
+	    if (feof(f)) {
+		clearerr(f);
+		goto end;
+	    }
+	    assert(ferror(f));
+	    switch (errno) {
+		case EINTR:
+		case EAGAIN:
+#if EAGAIN != EWOULDBLOCK
+		case EWOULDBLOCK:
+#endif
+		    clearerr(f);
+		    ok = wait_for_input(fd, true, -1);
+		    break;
+		default:
+		    xerror(errno, Ngt("cannot read input"));
+		    ok = false;
+		    break;
+	    }
 	}
     }
 end:
-    return (initlen == buf->length) ? EOF : 0;
+    if (initlen != buf->length)
+	return INPUT_OK;
+    else if (ok)
+	return INPUT_EOF;
+    else
+	return INPUT_ERROR;
 }
 
 /* An input function that reads input from the standard input.
@@ -201,11 +149,17 @@ end:
  * read after the newline.
  * This function does not use `inputinfo'.
  * The result is appended to the buffer. */
-int input_stdin(struct xwcsbuf_T *buf, void *inputinfo __attribute__((unused)))
+inputresult_T input_stdin(
+	struct xwcsbuf_T *buf, void *inputinfo __attribute__((unused)))
 {
     size_t initlen = buf->length;
-    read_line_from_stdin(buf, true);
-    return (initlen == buf->length) ? EOF : 0;
+    bool ok = read_line_from_stdin(buf, true);
+    if (initlen != buf->length)
+	return INPUT_OK;
+    else if (ok)
+	return INPUT_EOF;
+    else
+	return INPUT_ERROR;
 }
 
 /* Reads a line of input from the standard input.
@@ -237,7 +191,7 @@ bool read_line_from_stdin(struct xwcsbuf_T *buf, bool trap)
 #if EAGAIN != EWOULDBLOCK
 	    case EWOULDBLOCK:
 #endif
-		wait_for_input(STDIN_FILENO, trap, -1);
+		ok = wait_for_input(STDIN_FILENO, trap, -1);
 		break;
 	    default:
 		xerror(errno, Ngt("cannot read input"));
@@ -270,18 +224,21 @@ done:
     return ok;
 }
 
-/* An input function that prints a prompt and read input.
+/* An input function that prints a prompt and reads input.
  * `inputinfo' is a pointer to a `struct input_readline_info'.
  * `inputinfo->type' must be between 1 and 4 inclusive and specifies the type of
  * the prompt. For example, PS1 is printed if `inputinfo->type' is 1.
  * If `inputinfo->type' is 1, this function changes its value to 2. */
-int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
+inputresult_T input_readline(struct xwcsbuf_T *buf, void *inputinfo)
 {
 #if YASH_ENABLE_LINEEDIT
+    /* An input function must not return more than one line at a time.
+     * If line editing returns more than one line, this function returns only
+     * the first line, saving the rest in this buffer. */
     static wchar_t *linebuffer = NULL;
     if (linebuffer) {
 	linebuffer = forward_line(linebuffer, buf);
-	return 0;
+	return INPUT_OK;
     }
 #endif
 
@@ -292,7 +249,7 @@ int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
 	    exec_variable_as_commands(
 		    L VAR_PROMPT_COMMAND, VAR_PROMPT_COMMAND);
     if (do_job_control)
-	print_job_status_all(true, false, stderr);
+	print_job_status_all();
     if (info->type == 1)
 	check_mail();
     /* Note: no commands must be executed between `print_job_status_all' here
@@ -301,7 +258,7 @@ int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
      * called from `wait_for_input' in the line-editing. */
 
 #if YASH_ENABLE_LINEEDIT
-    /* read a line using line edit */
+    /* read a line using line editing */
     if (shopt_lineedit != shopt_nolineedit) {
 	unset_nonblocking(STDIN_FILENO);
 	if (le_setup()) {
@@ -318,19 +275,19 @@ int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
 			add_history(line);
 #endif /* YASH_ENABLE_HISTORY */
 		    linebuffer = forward_line(line, buf);
-		    return 0;
+		    return INPUT_OK;
 		} else {
 		    free(line);
-		    return EOF;
+		    return INPUT_EOF;
 		}
 	    } else {
-		return 1;
+		return INPUT_INTERRUPTED;
 	    }
 	}
     }
 #endif /* YASH_ENABLE_LINEEDIT */
 
-    /* read a line without using line edit */
+    /* read a line without line editing */
     print_prompt(info->type);
     if (info->type == 1)
 	info->type = 2;
@@ -414,7 +371,7 @@ just_print:
     }
 }
 
-/* Expands the contents of PS1 variable in the posixly correct way.
+/* Expands the contents of the PS1 variable in the posixly correct way.
  * The argument is `free'd in this function.
  * The return value must be `free'd by the caller. */
 /* In this function, "!" is expanded to the next history number and "!!" to "!"
@@ -491,21 +448,21 @@ wchar_t *expand_ps_yash(wchar_t *s)
 	if (*s != L'\\') {
 	    wb_wccat(&buf, *s);
 	} else switch (*++s) {
-	case L'\0':   wb_wccat(&buf, L'\\');     goto done;
-	//case L'\\':   wb_wccat(&buf, L'\\');     break;
-	case L'a':    wb_wccat(&buf, L'\a');     break;
-	case L'e':    wb_wccat(&buf, L'\033');   break;
-	case L'n':    wb_wccat(&buf, L'\n');     break;
-	case L'r':    wb_wccat(&buf, L'\r');     break;
-	default:      wb_wccat(&buf, *s);        break;
-	case L'$':    wb_wccat(&buf, geteuid() ? L'$' : L'#');  break;
-	case L'j':    wb_wprintf(&buf, L"%zu", job_count());  break;
+	    case L'\0':   wb_wccat(&buf, L'\\');     goto done;
+	  //case L'\\':   wb_wccat(&buf, L'\\');     break;
+	    case L'a':    wb_wccat(&buf, L'\a');     break;
+	    case L'e':    wb_wccat(&buf, L'\033');   break;
+	    case L'n':    wb_wccat(&buf, L'\n');     break;
+	    case L'r':    wb_wccat(&buf, L'\r');     break;
+	    default:      wb_wccat(&buf, *s);        break;
+	    case L'$':    wb_wccat(&buf, geteuid() ? L'$' : L'#');  break;
+	    case L'j':    wb_wprintf(&buf, L"%zu", job_count());  break;
 #if YASH_ENABLE_HISTORY
-	case L'!':    wb_wprintf(&buf, L"%d", hist_next_number); break;
+	    case L'!':    wb_wprintf(&buf, L"%d", hist_next_number); break;
 #endif
-	case L'f':    skip_alnum(&s);  break;
-	case L'[':    break;
-	case L']':    break;
+	    case L'f':    skip_alnum(&s);  break;
+	    case L'[':    break;
+	    case L']':    break;
 	}
 	s++;
     }
