@@ -32,24 +32,33 @@ struct xwcsbuf_T;
 extern _Bool read_line_from_stdin(struct xwcsbuf_T *buf, _Bool trap)
     __attribute__((nonnull));
 
-/* input functions to be used as `inputfunc_T' */
-extern int input_mbs(struct xwcsbuf_T *buf, void *inputinfo)
-    __attribute__((nonnull));
-extern int input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
-    __attribute__((nonnull));
-extern int input_file(struct xwcsbuf_T *buf, void *inputinfo)
-    __attribute__((nonnull));
-extern int input_stdin(struct xwcsbuf_T *buf, void *inputinfo)
-    __attribute__((nonnull(1)));
-extern int input_readline(struct xwcsbuf_T *buf, void *inputinfo)
-    __attribute__((nonnull));
 
-/* to be used as `inputinfo' for `input_mbs' */
-struct input_mbs_info {
-    const char *src;   /* the source code input */
-    size_t srclen;     /* # of bytes in `src' including trailing '\0' */
-    mbstate_t state;   /* shift state for multibyte-wide conversion */
-};
+/* The result type of `inputfunc_T'. */
+typedef enum inputresult_T {
+    INPUT_OK,           /* A line was read successfully. */
+    INPUT_EOF,          /* The end of file was reached. */
+    INPUT_INTERRUPTED,  /* SIGINT was received (interactive shell only) */
+    INPUT_ERROR,        /* Other error was encountered. */
+} inputresult_T;
+
+/* The type of input functions.
+ * Input is done line-wise: one line is read at a time.
+ * The input string is terminated by newline L'\n' except when there is no
+ * newline at the end of input.
+ * The result is indicated by a value of the `inputresult_T' type. If the return
+ * value is other than INPUT_OK, the buffer is unchanged.
+ * The input function may be called even after it returned a non-zero. */
+typedef inputresult_T inputfunc_T(struct xwcsbuf_T *buf, void *inputinfo);
+
+/* input functions to be used as `inputfunc_T' */
+extern inputresult_T input_wcs(struct xwcsbuf_T *buf, void *inputinfo)
+    __attribute__((nonnull));
+extern inputresult_T input_file(struct xwcsbuf_T *buf, void *inputinfo)
+    __attribute__((nonnull));
+extern inputresult_T input_stdin(struct xwcsbuf_T *buf, void *inputinfo)
+    __attribute__((nonnull(1)));
+extern inputresult_T input_readline(struct xwcsbuf_T *buf, void *inputinfo)
+    __attribute__((nonnull));
 
 /* to be used as `inputinfo' for `input_wcs' */
 struct input_wcs_info {
