@@ -51,8 +51,9 @@ static inline job_T *get_job(size_t jobnumber)
     __attribute__((pure));
 static inline void free_job(job_T *job);
 static void trim_joblist(void);
-static void set_current_jobnumber(size_t no);
+static void set_current_jobnumber(size_t jobnumber);
 static size_t find_next_job(size_t numlimit);
+static void apply_curstop(void);
 static int calc_status(int status)
     __attribute__((const));
 static wchar_t *get_job_name(const job_T *job)
@@ -71,11 +72,12 @@ static void jobs_builtin_print_job(size_t jobnumber,
 	bool runningonly, bool stoppedonly);
 static int continue_job(size_t jobnumber, job_T *job, bool fg)
     __attribute__((nonnull));
+static bool wait_has_job(bool jobcontrol);
 
 
 /* The list of jobs.
- * `joblist.contents[ACTIVE_JOBNO]' is a special job, which is called "active
- * job": the job that is being executed. */
+ * `joblist.contents[ACTIVE_JOBNO]' is a special job that is called "active
+ * job": the job that is currently being executed. */
 static plist_T joblist;
 
 /* number of the current/previous jobs. 0 if none. */
@@ -97,39 +99,40 @@ void set_active_job(job_T *job)
 }
 
 /* Moves the active job into the job list.
- * If `current' is true or there is no current job, the job will be the current
- * job. */
+ * If the newly added job is stopped, it becomes the current job.
+ * If `current' is true or there is no current job, the newly added job becomes
+ * the current job if there is no stopped job. */
 void add_job(bool current)
 {
     job_T *job = joblist.contents[ACTIVE_JOBNO];
+    size_t jobnumber;
 
     assert(job != NULL);
     joblist.contents[ACTIVE_JOBNO] = NULL;
 
     /* if there is an empty element in the list, use it */
-    for (size_t i = 1; i < joblist.length; i++) {
-	if (joblist.contents[i] == NULL) {
-	    joblist.contents[i] = job;
-	    if (current || current_jobnumber == 0)
-		set_current_jobnumber(i);
-	    else if (previous_jobnumber == 0)
-		previous_jobnumber = i;
-	    return;
+    for (jobnumber = 1; jobnumber < joblist.length; jobnumber++) {
+	if (joblist.contents[jobnumber] == NULL) {
+	    joblist.contents[jobnumber] = job;
+	    goto set_current;
 	}
     }
 
     /* if there is no empty, append at the end of the list */
     pl_add(&joblist, job);
-    if (current || current_jobnumber == 0)
-	set_current_jobnumber(joblist.length - 1);
-    else if (previous_jobnumber == 0)
-	previous_jobnumber = joblist.length - 1;
+
+set_current:
+    assert(joblist.contents[jobnumber] == job);
+    if (job->j_status == JS_STOPPED || current)
+	set_current_jobnumber(jobnumber);
+    else
+	set_current_jobnumber(current_jobnumber);
 }
 
 /* Returns the job of the specified number or NULL if not found. */
 job_T *get_job(size_t jobnumber)
 {
-    return jobnumber < joblist.length ? joblist.contents[jobnumber] : NULL;
+    return (jobnumber < joblist.length) ? joblist.contents[jobnumber] : NULL;
 }
 
 /* Removes the job of the specified number.
@@ -137,29 +140,24 @@ job_T *get_job(size_t jobnumber)
  * (another job is assigned to it). */
 void remove_job(size_t jobnumber)
 {
-    job_T *job = get_job(jobnumber);
+    free_job(get_job(jobnumber));
     joblist.contents[jobnumber] = NULL;
-    free_job(job);
     trim_joblist();
-
-    if (jobnumber == current_jobnumber) {
-	current_jobnumber = previous_jobnumber;
-	previous_jobnumber = find_next_job(current_jobnumber);
-    } else if (jobnumber == previous_jobnumber) {
-	previous_jobnumber = find_next_job(current_jobnumber);
-    }
+    set_current_jobnumber(current_jobnumber);
 }
 
 /* Removes all jobs unconditionally. */
 void remove_all_jobs(void)
 {
-    for (size_t i = 0; i < joblist.length; i++)
-	remove_job(i);
+    for (size_t i = 0; i < joblist.length; i++) {
+	free_job(joblist.contents[i]);
+	joblist.contents[i] = NULL;
+    }
     trim_joblist();
     current_jobnumber = previous_jobnumber = 0;
 }
 
-/* Frees a job. */
+/* Frees the specified job. */
 void free_job(job_T *job)
 {
     if (job) {
@@ -176,6 +174,7 @@ void trim_joblist(void)
 
     while (tail > 0 && joblist.contents[--tail] == NULL);
     tail++;
+    assert(tail > 0 && joblist.contents[tail] == NULL);
     if (joblist.maxlength > 20 && joblist.maxlength / 2 > joblist.length)
 	pl_setmax(&joblist, tail);
     else
@@ -194,42 +193,83 @@ void neglect_all_jobs(void)
     current_jobnumber = previous_jobnumber = 0;
 }
 
-/* - When the current job changes, the last current job will be the next
- *   previous job.
- *   - The "fg" command changes the current job.
- *   - The `add_job' function may change the current job.
- * - When the current job finishes, the previous job becomes the current job.
- * - Restarting the current or previous job by the "bg" command resets the
- *   current and previous jobs.
+/* Current/previous job selection discipline:
+ *
+ * - When there is one or more stopped jobs, the current job must be one of
+ *   them.
+ * - When there are more than one stopped job, the previous job must be one of
+ *   them but the current one.
+ * - The current job becomes the previous job when another job becomes the
+ *   current.
+ *
+ * - When a foreground job is stopped, it becomes the current job.
+ * - When an asynchronous command is executed and the "curasync" option is set,
+ *   it becomes the current job.
+ * - When a job is continued by the "bg" command and the "curbg" option is set,
+ *   it becomes the current job.
+ * - When a job is stopped and the "curstop" option is set, it becomes the
+ *   current job.
+ *
  * - The "wait" command doesn't change the current and previous jobs. */
 
 /* Sets the current job number to the specified one and resets the previous job
- * number. If `jobnumber' is 0, the previous job becomes the current job.
- * Otherwise `jobnumber' must be a valid job number. */
+ * number. If the specified job number is not used, a job is arbitrarily chosen
+ * for the current. If there is one or more stopped jobs and the one specified
+ * by the argument is not stopped, the current job is not changed. */
+/* This function must be called whenever a job is added to or removed from the
+ * job list or any job's status has been changed. */
 void set_current_jobnumber(size_t jobnumber)
 {
-    assert(jobnumber == 0 || get_job(jobnumber) != NULL);
+    size_t stopcount = stopped_job_count();
+    const job_T *newcurrent = get_job(jobnumber);
 
-    previous_jobnumber = current_jobnumber;
-    if (jobnumber == 0) {
-	jobnumber = previous_jobnumber;
-	if (jobnumber == 0 || get_job(jobnumber) == NULL)
-	    jobnumber = find_next_job(0);
+    if (newcurrent == NULL
+	    || (stopcount > 0 && newcurrent->j_status != JS_STOPPED)) {
+	jobnumber = current_jobnumber;
+	newcurrent = get_job(jobnumber);
+	if (newcurrent == NULL
+		|| (stopcount > 0 && newcurrent->j_status != JS_STOPPED)) {
+	    jobnumber = previous_jobnumber;
+	    newcurrent = get_job(jobnumber);
+	    if (newcurrent == NULL
+		    || (stopcount > 0 && newcurrent->j_status != JS_STOPPED))
+		jobnumber = find_next_job(0);
+	}
     }
-    current_jobnumber = jobnumber;
 
-    if (previous_jobnumber == 0
-	    || previous_jobnumber == current_jobnumber)
-	previous_jobnumber = find_next_job(current_jobnumber);
+    if (jobnumber != current_jobnumber) {
+	size_t oldcurrentnum = current_jobnumber;
+	current_jobnumber = jobnumber;
+	jobnumber = oldcurrentnum;
+    } else {
+	jobnumber = previous_jobnumber;
+    }
+
+    const job_T *newprevious = get_job(jobnumber);
+
+    if (newprevious == NULL || jobnumber == current_jobnumber
+	    || (stopcount > 1 && newprevious->j_status != JS_STOPPED)) {
+	jobnumber = previous_jobnumber;
+	newprevious = get_job(jobnumber);
+	if (newprevious == NULL || jobnumber == current_jobnumber
+		|| (stopcount > 1 && newprevious->j_status != JS_STOPPED))
+	    jobnumber = find_next_job(current_jobnumber);
+    }
+    previous_jobnumber = jobnumber;
 }
 
 /* Returns an arbitrary job number except the specified.
- * The returned number is suitable for the next current/previous jobs.
+ * The returned number is suitable for the current/previous jobs.
  * If there is no job to pick out, 0 is returned.
  * Stopped jobs are preferred to running/finished jobs.
- * If there are more than one stopped jobs, larger job number is preferred. */
+ * If there are more than one stopped jobs, the previous job is preferred. */
 size_t find_next_job(size_t excl)
 {
+    if (previous_jobnumber != excl) {
+	job_T *job = get_job(previous_jobnumber);
+	if (job != NULL && job->j_status == JS_STOPPED)
+	    return previous_jobnumber;
+    }
     size_t jobnumber = joblist.length;
     while (--jobnumber > 0) {
 	if (jobnumber != excl) {
@@ -249,6 +289,20 @@ size_t find_next_job(size_t excl)
     return 0;
 }
 
+/* If the "curstop" option is set and there is a job which has been stopped and
+ * whose `j_statuschanged' flag is set, make it the current job. */
+void apply_curstop(void)
+{
+    if (shopt_curstop) {
+	for (size_t i = 0; i < joblist.length; i++) {
+	    job_T *job = joblist.contents[i];
+	    if (job && job->j_status == JS_STOPPED && job->j_statuschanged)
+		set_current_jobnumber(i);
+	}
+    }
+    set_current_jobnumber(current_jobnumber);
+}
+
 /* Counts the number of jobs in the job list. */
 size_t job_count(void)
 {
@@ -272,7 +326,7 @@ size_t stopped_job_count(void)
 }
 
 
-/* Updates the info about the jobs in the job list by calling `waitpid'.
+/* Updates the info about the jobs in the job list.
  * This function doesn't block. */
 void do_wait(void)
 {
@@ -291,7 +345,6 @@ start:
 		goto start;  /* try again */
 	    case ECHILD:
 		return;      /* there are no child processes */
-		/* falls thru! */
 	    default:
 		xerror(errno, "waitpid");
 		return;
@@ -313,7 +366,7 @@ start:
 		    goto found;
 
     /* If `pid' is not found in the job list, we simply ignore it. This may
-     * happen on some occasions: e.g. the job is "disown"ed. */
+     * happen on some occasions: e.g. the job has been "disown"ed. */
     goto start;
 
 found:
@@ -351,7 +404,7 @@ out_of_loop:
     goto start;
 }
 
-/* Waits for a job to finish (or stop).
+/* Waits for the specified job to finish (or stop).
  * `jobnumber' must be a valid job number.
  * If `return_on_stop' is false, waits for the job to finish.
  * Otherwise, waits for the job to finish or stop.
@@ -387,9 +440,9 @@ int wait_for_job(size_t jobnumber, bool return_on_stop,
     return signum;
 }
 
-/* Waits for a child process to finish (or stop).
- * `cpid' is the process ID of the child process to wait for, which is not yet
- * registered in the job list.
+/* Waits for the specified child process to finish (or stop).
+ * `cpid' is the process ID of the child process to wait for. This must not be
+ * in the job list.
  * `cpgid' is the process group ID of the child. If the child's PGID is the same
  * as that of the parent, `cpgid' must be 0.
  * If `return_on_stop' is false, waits for the job to finish.
@@ -482,8 +535,8 @@ void put_foreground(pid_t pgrp)
 void ensure_foreground(void)
 {
     /* This function calls `tcsetpgrp' with the default SIGTTOU handler. If the
-     * shell is in the background, it will receive SIGTTOU and is stopped until
-     * continued in the foreground. */
+     * shell is in the background, it will receive SIGTTOU and get stopped until
+     * it is continued in the foreground. */
 
     struct sigaction dflsa, savesa;
     sigset_t blockss, savess;
@@ -525,7 +578,7 @@ int calc_status(int status)
 }
 
 /* Computes the exit status of the specified job.
- * The job must be JS_DONE or JS_STOPPED. */
+ * The job state must be JS_DONE or JS_STOPPED. */
 int calc_status_of_job(const job_T *job)
 {
     switch (job->j_status) {
@@ -633,12 +686,13 @@ char *get_job_status_string(const job_T *job, bool *needfree)
     assert(false);
 }
 
-/* Prints the status of job(s).
+/* Prints the status of the specified job.
  * Finished jobs are removed from the job list after the status is printed.
  * If the specified job doesn't exist, nothing is printed (it isn't an error).
- * If `changedonly' is true, only jobs whose `j_statuschanged' is true is
- * printed. If `verbose' is true, the status is printed in the process-wise
- * format rather than the usual job-wise format. */
+ * If `changedonly' is true, the job is printed only if the `j_statuschanged'
+ * flag is true.
+ * If `verbose' is true, the status is printed in the process-wise format rather
+ * than the usual job-wise format. */
 void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 {
     job_T *job = get_job(jobnumber);
@@ -700,11 +754,12 @@ void print_job_status(size_t jobnumber, bool changedonly, bool verbose, FILE *f)
 	remove_job(jobnumber);
 }
 
-/* Prints the status of all jobs. */
-void print_job_status_all(bool changedonly, bool verbose, FILE *f)
+/* Prints the status of jobs which have been changed but not reported. */
+void print_job_status_all(void)
 {
+    apply_curstop();
     for (size_t i = 1; i < joblist.length; i++)
-	print_job_status(i, changedonly, verbose, f);
+	print_job_status(i, true, false, stderr);
 }
 
 /* If the shell is interactive and the specified job has been killed by a
@@ -816,13 +871,13 @@ found:
 
 /********** Builtins **********/
 
-/* "jobs" builtin, which accepts the following options:
- * -l: be verbose
- * -n: print the jobs only whose status have changed
- * -p: print the process ID only
- * -r: print running jobs only
- * -s: print stopped jobs only
- * If `posixly_correct' is true, only -l and -p are available. */
+/* The "jobs" builtin, which accepts the following options:
+ *  -l: be verbose
+ *  -n: print the jobs only whose status have changed
+ *  -p: print the process ID only
+ *  -r: print running jobs only
+ *  -s: print stopped jobs only
+ * In the POSIXly correct mode, only -l and -p are available. */
 int jobs_builtin(int argc, void **argv)
 {
     static const struct xoption long_options[] = {
@@ -831,7 +886,9 @@ int jobs_builtin(int argc, void **argv)
 	{ L"pgid-only",    xno_argument, L'p', },
 	{ L"running-only", xno_argument, L'r', },
 	{ L"stopped-only", xno_argument, L's', },
+#if YASH_ENABLE_HELP
 	{ L"help",         xno_argument, L'-', },
+#endif
 	{ NULL, 0, 0, },
     };
 
@@ -850,8 +907,10 @@ int jobs_builtin(int argc, void **argv)
 	    case L'p':  pgidonly    = true;  break;
 	    case L'r':  runningonly = true;  break;
 	    case L's':  stoppedonly = true;  break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
 		fprintf(stderr, gt(posixly_correct
 			    ? Ngt("Usage:  jobs [-lp] [job...]\n")
@@ -861,9 +920,11 @@ int jobs_builtin(int argc, void **argv)
     }
 
     nextforceexit = true;
+    apply_curstop();
 
     clearerr(stdout);
     if (xoptind < argc) {
+	/* print the specified jobs */
 	do {
 	    const wchar_t *jobspec = ARGV(xoptind);
 	    if (jobspec[0] == L'%') {
@@ -925,6 +986,7 @@ void jobs_builtin_print_job(size_t jobnumber,
     }
 }
 
+#if YASH_ENABLE_HELP
 const char jobs_help[] = Ngt(
 "jobs - print info about jobs\n"
 "\tjobs [-lnprs] [job...]\n"
@@ -935,7 +997,7 @@ const char jobs_help[] = Ngt(
 " -l --verbose\n"
 "\tprint info for each process in the job, including process ID\n"
 " -n --new\n"
-"\tprint jobs whose status have changed only\n"
+"\tonly print jobs whose status have changed\n"
 " -p --pgid-only\n"
 "\tprint process group IDs only\n"
 " -r --running-only\n"
@@ -944,8 +1006,9 @@ const char jobs_help[] = Ngt(
 "\tprint stopped jobs only\n"
 "In POSIXly correct mode, only the -l and -p options are available.\n"
 );
+#endif
 
-/* "fg"/"bg" builtin */
+/* The "fg"/"bg" builtin */
 int fg_builtin(int argc, void **argv)
 {
     bool fg = wcscmp(argv[0], L"fg") == 0;
@@ -955,8 +1018,10 @@ int fg_builtin(int argc, void **argv)
     xoptind = 0, xopterr = true;
     while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
 	switch (opt) {
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr, gt(fg ? Ngt("Usage:  fg [job]\n")
 		                      : Ngt("Usage:  bg [job...]\n")));
@@ -1000,7 +1065,6 @@ int fg_builtin(int argc, void **argv)
 		xerror(0, Ngt("%ls: not job-controlled job"), ARGV(xoptind));
 		err = true;
 	    } else {
-		set_current_jobnumber(jobnumber);
 		status = continue_job(jobnumber, job, fg);
 	    }
 	} while (++xoptind < argc);
@@ -1053,13 +1117,21 @@ int continue_job(size_t jobnumber, job_T *job, bool fg)
 	if (termsave)
 	    le_restore_terminal();
 #endif
-	status = (job->j_status == JS_RUNNING)
-	    ? Exit_SUCCESS : calc_status_of_job(job);
-	if (job->j_status == JS_DONE) {
-	    notify_signaled_job(jobnumber);
-	    remove_job(jobnumber);
+	switch (job->j_status) {
+	    case JS_STOPPED:
+		status = calc_status_of_job(job);
+		set_current_jobnumber(jobnumber);
+		break;
+	    case JS_DONE:
+		status = calc_status_of_job(job);
+		notify_signaled_job(jobnumber);
+		remove_job(jobnumber);
+		break;
+	    default:
+		assert(false);
 	}
     } else {
+	set_current_jobnumber(shopt_curbg ? jobnumber : current_jobnumber);
 	status = (job->j_status == JS_RUNNING) ? Exit_SUCCESS : Exit_FAILURE;
     }
     return status;
@@ -1070,13 +1142,15 @@ int continue_job(size_t jobnumber, job_T *job, bool fg)
      * invoked in the background.
      * Programs that change the terminal state generally save the state before
      * changing it and restore it when they are finished. But, if they are
-     * invoked in the background, they save the state that is possibly being
-     * used by another program (typically the shell's line-editing), so the
-     * state that they restore is not the normal state.
+     * invoked in the background, they may save the state that is being used by
+     * another program (typically the shell's line-editing), so the state that
+     * they restore is not the normal state.
      * The shell tackles this problem by saving and restoring the terminal state
      * for the continued programs. */
 }
 
+#if YASH_ENABLE_HELP
+
 const char fg_help[] = Ngt(
 "fg - run jobs in the foreground\n"
 "\tfg [job...]\n"
@@ -1093,7 +1167,9 @@ const char bg_help[] = Ngt(
 "If no job is specified, the current job is continued.\n"
 );
 
-/* "wait" builtin */
+#endif /* YASH_ENABLE_HELP */
+
+/* The "wait" builtin */
 int wait_builtin(int argc, void **argv)
 {
     bool jobcontrol = doing_job_control_now;
@@ -1104,8 +1180,10 @@ int wait_builtin(int argc, void **argv)
     xoptind = 0, xopterr = true;
     while ((opt = xgetopt_long(argv, L"", help_option, NULL))) {
 	switch (opt) {
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
 		fprintf(stderr, gt("Usage:  wait [job or pid...]\n"));
 		return Exit_ERROR;
@@ -1114,6 +1192,7 @@ int wait_builtin(int argc, void **argv)
 
     job_T *job;
     if (xoptind < argc) {
+	/* wait for the specified jobs */
 	do {
 	    const wchar_t *jobspec = ARGV(xoptind);
 	    size_t jobnumber;
@@ -1142,7 +1221,7 @@ int wait_builtin(int argc, void **argv)
 		if (!status) {
 		    status = calc_status_of_job(job);
 		} else {
-		    assert(TERMSIGOFFSET > 128);
+		    assert(TERMSIGOFFSET >= 128);
 		    status += TERMSIGOFFSET;
 		    break;
 		}
@@ -1156,21 +1235,12 @@ int wait_builtin(int argc, void **argv)
 	} while (++xoptind < argc);
     } else {
 	/* wait for all jobs */
-	for (size_t i = 1; i < joblist.length; i++) {
-	    job = joblist.contents[i];
-	    if (job != NULL && job->j_pgid >= 0) {
-		status = wait_for_job(i, jobcontrol, jobcontrol, true);
-		if (status) {
-		    assert(TERMSIGOFFSET > 128);
-		    status += TERMSIGOFFSET;
-		    break;
-		}
-		if (job->j_status != JS_RUNNING) {
-		    if (jobcontrol && is_interactive_now && !posixly_correct)
-			print_job_status(i, false, false, stdout);
-		    else if (job->j_status == JS_DONE)
-			remove_job(i);
-		}
+	while (wait_has_job(jobcontrol)) {
+	    status = wait_for_sigchld(jobcontrol, true);
+	    if (status) {
+		assert(TERMSIGOFFSET >= 128);
+		status += TERMSIGOFFSET;
+		break;
 	    }
 	}
     }
@@ -1178,6 +1248,29 @@ int wait_builtin(int argc, void **argv)
     return status ? status : err ? Exit_FAILURE : Exit_SUCCESS;
 }
 
+/* Checks if the shell has any job to wait for. */
+bool wait_has_job(bool jobcontrol)
+{
+    /* print/remove already-finished jobs */
+    if (jobcontrol && is_interactive_now && !posixly_correct) {
+	print_job_status_all();
+    } else {
+	for (size_t i = 1; i < joblist.length; i++) {
+	    job_T *job = joblist.contents[i];
+	    if (job != NULL && (job->j_pgid < 0 || job->j_status == JS_DONE))
+		remove_job(i);
+	}
+    }
+
+    for (size_t i = 1; i < joblist.length; i++) {
+	job_T *job = joblist.contents[i];
+	if (job != NULL && (!jobcontrol || job->j_status == JS_RUNNING))
+	    return true;
+    }
+    return false;
+}
+
+#if YASH_ENABLE_HELP
 const char wait_help[] = Ngt(
 "wait - wait for jobs to terminate\n"
 "\twait [job or pid...]\n"
@@ -1185,29 +1278,26 @@ const char wait_help[] = Ngt(
 "Jobs can be specified in the usual job specification form such as \"%2\" or\n"
 "by the process ID of a process belonging to the job.\n"
 );
+#endif
 
-/* "disown" builtin, which accepts the following option:
- * -a: disown all jobs */
+/* The "disown" builtin, which accepts the following option:
+ *  -a: disown all jobs */
 int disown_builtin(int argc, void **argv)
 {
-    static const struct xoption long_options[] = {
-	{ L"all",  xno_argument, L'a', },
-	{ L"help", xno_argument, L'-', },
-	{ NULL, 0, 0, },
-    };
-
     bool all = false;
     bool err = false;
     wchar_t opt;
 
     xoptind = 0, xopterr = true;
-    while ((opt = xgetopt_long(argv, L"a", long_options, NULL))) {
+    while ((opt = xgetopt_long(argv, L"a", all_option, NULL))) {
 	switch (opt) {
 	    case L'a':  all = true;  break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:
-		fprintf(stderr, gt("Usage:  disown [job...]\n"));
+		fprintf(stderr, gt("Usage:  disown [-a] [job...]\n"));
 		return Exit_ERROR;
 	}
     }
@@ -1248,6 +1338,7 @@ int disown_builtin(int argc, void **argv)
     return err ? Exit_FAILURE : Exit_SUCCESS;
 }
 
+#if YASH_ENABLE_HELP
 const char disown_help[] = Ngt(
 "disown - disown jobs\n"
 "\tdisown [job...]\n"
@@ -1259,6 +1350,7 @@ const char disown_help[] = Ngt(
 "Otherwise, the specified jobs are disowned. If none is specified, the\n"
 "current job is disowned.\n"
 );
+#endif
 
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
