@@ -48,7 +48,7 @@ extern void le_display_size_changed(void);
 
 extern _Bool le_next_verbatim;
 
-void append_to_prebuffer(char *s)
+extern void append_to_prebuffer(char *s)
     __attribute__((nonnull));
 
 
