@@ -56,83 +56,88 @@ static void set_le_noconvmeta_option(void *argp);
 
 
 /* If set, the shell behaves strictly as defined in POSIX.
- * Corresponds to --posix option. */
+ * Corresponds to the --posix option. */
 bool posixly_correct;
 
 /* If set, this shell is a login shell.
- * Corresponds to --login option. */
+ * Corresponds to the --login option. */
 bool is_login_shell;
 
 /* If set, this shell is interactive.
  * `is_interactive_now' is set false in subshells while `is_interactive'
  * remains unchanged.
- * Correspond to -i/--interactive option. */
+ * Once initialized, `is_interactive' cannot be reset.
+ * Correspond to the -i/--interactive option. */
 bool is_interactive, is_interactive_now;
 
 /* If set, the shell performs job control.
- * Corresponds to -m/--monitor option. */
+ * Corresponds to the -m/--monitor option. */
 bool do_job_control;
-/* If set, the shell immediately notifies on a change of job status.
- * Corresponds to -b/--notify option. */
+/* If set, the shell immediately notifies on change of job status.
+ * Corresponds to the -b/--notify option. */
 bool shopt_notify;
 #if YASH_ENABLE_LINEEDIT
-/* If set, the shell immediately notifies on a change of job status during line-
- * editing. Ignored if `shopt_notify' is set. Corresponds to --notifyle option.
- */
+/* If set, the shell immediately notifies on change of job status during line-
+ * editing. Ignored if `shopt_notify' is set. Corresponds to the --notifyle
+ * option. */
 bool shopt_notifyle;
 #endif
 
 /* Set if commands are read from argument and stdin respectively.
- * Correspond to -c and -s options respectively. */
+ * Correspond to the -c and -s options respectively. */
 bool shopt_read_arg, shopt_read_stdin;
 
 /* The value of special parameter $0. */
 const wchar_t *command_name;
 
 /* If set, any variable is exported when assigned.
- * Corresponds to -a/--allexport option. */
+ * Corresponds to the -a/--allexport option. */
 bool shopt_allexport;
 /* If set, when a function is defined, all the commands in the function
- * are hashed. Corresponds to -h/--hashondef option. */
+ * are hashed. Corresponds to the -h/--hashondef option. */
 bool shopt_hashondef;
 
-/* If set, when a command returns a non-zero status, exit the shell.
- * Corresponds to -e/--errexit option. */
+/* If set, when a command returns a non-zero status, the shell exits.
+ * Corresponds to the -e/--errexit option. */
 bool shopt_errexit;
-/* If set, treat an expansion of undefined parameter as an error.
- * Corresponds to -u/--nounset option. */
+/* If set, treat expansion of an undefined parameter as an error.
+ * Corresponds to the -u/--nounset option. */
 bool shopt_nounset;
 /* If set, don't execute any command; just do syntax checking.
- * Corresponds to -n/--noexec option. */
+ * Corresponds to the -n/--noexec option. */
 bool shopt_noexec;
-/* If set, don't exit when EOF is entered.
- * Corresponds to --ignoreeof option. */
+/* If set, don't exit when EOF is entered (if the input is from a terminal).
+ * Corresponds to the --ignoreeof option. */
 bool shopt_ignoreeof;
 /* If set, echo the input to the shell.
- * Corresponds to -v/--verbose option. */
+ * Corresponds to the -v/--verbose option. */
 bool shopt_verbose;
-/* If set, print the trace of each command executed and variable assigned.
- * Corresponds to -x/--xtrace option. */
+/* If set, print the trace of command execution and variable assignment.
+ * Corresponds to the -x/--xtrace option. */
 bool shopt_xtrace;
-/* If set, a new asynchronous job is set to the current job when invoked. */
-bool shopt_curasync;
+/* If set, a background job is set to the current job when:
+ *  - invoked as an asynchronous command
+ *  - resumed in the background by the "bg" command
+ *  - stopped
+ * respectively. */
+bool shopt_curasync = true, shopt_curbg = true, shopt_curstop = true;
 /* If set, lines that start with a space are not saved in the history. */
 bool shopt_histspace;
 
-/* If set, don't perform filename expansions.
- * Corresponds to -f/--noglob option. */
+/* If set, don't perform filename expansion.
+ * Corresponds to the -f/--noglob option. */
 bool shopt_noglob;
 /* Correspond to WGLB_CASEFOLD, WGLB_PERIOD, WGLB_MARK, WGLB_RECDIR resp. */
 bool shopt_nocaseglob, shopt_dotglob, shopt_markdirs, shopt_extendedglob;
-/* If set, globbing pattern is removed from command line rather than left
- * intact when there are no matches.
- * Corresponds to --nullglob option. */
+/* If set, a glob pattern is removed from the command line rather than left
+ * intact when there are no matches for it.
+ * Corresponds to the --nullglob option. */
 bool shopt_nullglob;
 /* If set, perform brace expansion.
- * Corresponds to --brace option.*/
+ * Corresponds to the --brace option.*/
 bool shopt_braceexpand;
 /* If set, prevent redirections from overwriting existent file.
- * Corresponds to -C/--noclobber option. */
+ * Corresponds to the -C/--noclobber option. */
 bool shopt_noclobber;
 
 #if YASH_ENABLE_LINEEDIT
@@ -140,6 +145,8 @@ bool shopt_noclobber;
 enum shopt_lineedit_T shopt_lineedit = shopt_nolineedit;
 /* Defines treatment of the 8th bit of input characters. */
 enum shopt_yesnoauto_T shopt_le_convmeta = shopt_auto;
+/* If set, line-editing uses a flash (instead of a bell) to alert the user. */
+bool shopt_le_visiblebell = false;
 /* If set, a special character sequence is printed when starting line-editing
  * to make sure the prompt starts at the beginning of line. */
 bool shopt_le_promptsp = true;
@@ -151,7 +158,8 @@ typedef enum shopt_index_T {
     SHOPT_RCFILE, SHOPT_VERSION,
     SHOPT_ALLEXPORT, SHOPT_HASHONDEF, SHOPT_NOCLOBBER, SHOPT_NOGLOB,
     SHOPT_NOCASEGLOB, SHOPT_DOTGLOB, SHOPT_MARKDIRS, SHOPT_EXTENDEDGLOB,
-    SHOPT_NULLGLOB, SHOPT_BRACEEXPAND, SHOPT_CURASYNC,
+    SHOPT_NULLGLOB, SHOPT_BRACEEXPAND, SHOPT_CURASYNC, SHOPT_CURBG,
+    SHOPT_CURSTOP,
     SHOPT_ERREXIT, SHOPT_NOUNSET, SHOPT_NOEXEC, SHOPT_IGNOREEOF, SHOPT_VERBOSE,
     SHOPT_XTRACE, SHOPT_HISTSPACE, SHOPT_NOLOG, SHOPT_MONITOR, SHOPT_NOTIFY,
 #if YASH_ENABLE_LINEEDIT
@@ -160,99 +168,121 @@ typedef enum shopt_index_T {
     SHOPT_POSIX,
 #if YASH_ENABLE_LINEEDIT
     SHOPT_VI, SHOPT_EMACS,
-    SHOPT_LE_CONVMETA, SHOPT_LE_NOCONVMETA, SHOPT_LE_PROMPTSP,
+    SHOPT_LE_CONVMETA, SHOPT_LE_NOCONVMETA,
+    SHOPT_LE_VISIBLEBELL, SHOPT_LE_PROMPTSP,
 #endif
+#if YASH_ENABLE_HELP
     SHOPT_HELP,
+#endif
+    SHOPT_end,
     SHOPT_setopt = SHOPT_ALLEXPORT,
 } shopt_index_T;
 
 
 /* Long options for the shell and set builtin */
 static const struct xoption long_options[] = {
-    [SHOPT_INTERACTIVE]   = { L"interactive",  xno_argument, L'i', },
-    [SHOPT_LOGIN]         = { L"login",        xno_argument, L'l', },
-    [SHOPT_NOPROFILE]     = { L"noprofile",    xno_argument, L'(', },
-    [SHOPT_NORCFILE]      = { L"norcfile",     xno_argument, L')', },
-    [SHOPT_RCFILE]        = { L"rcfile",       xrequired_argument, L'!', },
-    [SHOPT_VERSION]       = { L"version",      xno_argument, L'V', },
-    /* Options above cannot be used in set builtin */
-    [SHOPT_ALLEXPORT]     = { L"allexport",    xno_argument, L'L', },
-    [SHOPT_HASHONDEF]     = { L"hashondef",    xno_argument, L'L', },
-    [SHOPT_NOCLOBBER]     = { L"noclobber",    xno_argument, L'L', },
-    [SHOPT_NOGLOB]        = { L"noglob",       xno_argument, L'L', },
-    [SHOPT_NOCASEGLOB]    = { L"nocaseglob",   xno_argument, L'L', },
-    [SHOPT_DOTGLOB]       = { L"dotglob",      xno_argument, L'L', },
-    [SHOPT_MARKDIRS]      = { L"markdirs",     xno_argument, L'L', },
-    [SHOPT_EXTENDEDGLOB]  = { L"extendedglob", xno_argument, L'L', },
-    [SHOPT_NULLGLOB]      = { L"nullglob",     xno_argument, L'L', },
-    [SHOPT_BRACEEXPAND]   = { L"braceexpand",  xno_argument, L'L', },
-    [SHOPT_CURASYNC]      = { L"curasync",     xno_argument, L'L', },
-    [SHOPT_ERREXIT]       = { L"errexit",      xno_argument, L'L', },
-    [SHOPT_NOUNSET]       = { L"nounset",      xno_argument, L'L', },
-    [SHOPT_NOEXEC]        = { L"noexec",       xno_argument, L'L', },
-    [SHOPT_IGNOREEOF]     = { L"ignoreeof",    xno_argument, L'L', },
-    [SHOPT_VERBOSE]       = { L"verbose",      xno_argument, L'L', },
-    [SHOPT_XTRACE]        = { L"xtrace",       xno_argument, L'L', },
-    [SHOPT_HISTSPACE]     = { L"histspace",    xno_argument, L'L', },
-    [SHOPT_NOLOG]         = { L"nolog",        xno_argument, L'L', },
-    [SHOPT_MONITOR]       = { L"monitor",      xno_argument, L'm', },
-    [SHOPT_NOTIFY]        = { L"notify",       xno_argument, L'L', },
+    [SHOPT_INTERACTIVE]    = { L"interactive",    xno_argument, L'i', },
+    [SHOPT_LOGIN]          = { L"login",          xno_argument, L'l', },
+    [SHOPT_NOPROFILE]      = { L"noprofile",      xno_argument, L'(', },
+    [SHOPT_NORCFILE]       = { L"norcfile",       xno_argument, L')', },
+    [SHOPT_RCFILE]         = { L"rcfile",         xrequired_argument, L'!', },
+    [SHOPT_VERSION]        = { L"version",        xno_argument, L'V', },
+    /* Above options cannot be used thru the "set" builtin */
+    [SHOPT_ALLEXPORT]      = { L"allexport",      xno_argument, L'L', },
+    [SHOPT_HASHONDEF]      = { L"hashondef",      xno_argument, L'L', },
+    [SHOPT_NOCLOBBER]      = { L"noclobber",      xno_argument, L'L', },
+    [SHOPT_NOGLOB]         = { L"noglob",         xno_argument, L'L', },
+    [SHOPT_NOCASEGLOB]     = { L"nocaseglob",     xno_argument, L'L', },
+    [SHOPT_DOTGLOB]        = { L"dotglob",        xno_argument, L'L', },
+    [SHOPT_MARKDIRS]       = { L"markdirs",       xno_argument, L'L', },
+    [SHOPT_EXTENDEDGLOB]   = { L"extendedglob",   xno_argument, L'L', },
+    [SHOPT_NULLGLOB]       = { L"nullglob",       xno_argument, L'L', },
+    [SHOPT_BRACEEXPAND]    = { L"braceexpand",    xno_argument, L'L', },
+    [SHOPT_CURASYNC]       = { L"curasync",       xno_argument, L'L', },
+    [SHOPT_CURBG]          = { L"curbg",          xno_argument, L'L', },
+    [SHOPT_CURSTOP]        = { L"curstop",        xno_argument, L'L', },
+    [SHOPT_ERREXIT]        = { L"errexit",        xno_argument, L'L', },
+    [SHOPT_NOUNSET]        = { L"nounset",        xno_argument, L'L', },
+    [SHOPT_NOEXEC]         = { L"noexec",         xno_argument, L'L', },
+    [SHOPT_IGNOREEOF]      = { L"ignoreeof",      xno_argument, L'L', },
+    [SHOPT_VERBOSE]        = { L"verbose",        xno_argument, L'L', },
+    [SHOPT_XTRACE]         = { L"xtrace",         xno_argument, L'L', },
+    [SHOPT_HISTSPACE]      = { L"histspace",      xno_argument, L'L', },
+    [SHOPT_NOLOG]          = { L"nolog",          xno_argument, L'L', },
+    [SHOPT_MONITOR]        = { L"monitor",        xno_argument, L'm', },
+    [SHOPT_NOTIFY]         = { L"notify",         xno_argument, L'L', },
 #if YASH_ENABLE_LINEEDIT
-    [SHOPT_NOTIFYLE]      = { L"notifyle",     xno_argument, L'L', },
+    [SHOPT_NOTIFYLE]       = { L"notifyle",       xno_argument, L'L', },
 #endif
-    [SHOPT_POSIX]         = { L"posix",        xno_argument, L'L', },
+    [SHOPT_POSIX]          = { L"posix",          xno_argument, L'L', },
 #if YASH_ENABLE_LINEEDIT
-    [SHOPT_VI]            = { L"vi",           xno_argument, L'L', },
-    [SHOPT_EMACS]         = { L"emacs",        xno_argument, L'L', },
-    [SHOPT_LE_CONVMETA]   = { L"le-convmeta",  xno_argument, L'L', },
-    [SHOPT_LE_NOCONVMETA] = { L"le-noconvmeta",xno_argument, L'L', },
-    [SHOPT_LE_PROMPTSP]   = { L"le-promptsp",  xno_argument, L'L', },
+    [SHOPT_VI]             = { L"vi",             xno_argument, L'L', },
+    [SHOPT_EMACS]          = { L"emacs",          xno_argument, L'L', },
+    [SHOPT_LE_CONVMETA]    = { L"le-convmeta",    xno_argument, L'L', },
+    [SHOPT_LE_NOCONVMETA]  = { L"le-noconvmeta",  xno_argument, L'L', },
+    [SHOPT_LE_VISIBLEBELL] = { L"le-visiblebell", xno_argument, L'L', },
+    [SHOPT_LE_PROMPTSP]    = { L"le-promptsp",    xno_argument, L'L', },
 #endif
-    [SHOPT_HELP]          = { L"help",         xno_argument, L'-', },
-    /* this one must be the last for `help_option' */
-    { NULL, 0, 0, },
+#if YASH_ENABLE_HELP
+    [SHOPT_HELP]           = { L"help",         xno_argument, L'-', },
+#endif
+    [SHOPT_end]            = { NULL, 0, 0, },
 };
 
 static const struct setoptinfo_T setoptinfo[] = {
-    [SHOPT_ALLEXPORT]     = { set_bool_option, &shopt_allexport, },
-    [SHOPT_HASHONDEF]     = { set_bool_option, &shopt_hashondef, },
-    [SHOPT_NOCLOBBER]     = { set_bool_option, &shopt_noclobber, },
-    [SHOPT_NOGLOB]        = { set_bool_option, &shopt_noglob, },
-    [SHOPT_NOCASEGLOB]    = { set_bool_option, &shopt_nocaseglob, },
-    [SHOPT_DOTGLOB]       = { set_bool_option, &shopt_dotglob, },
-    [SHOPT_MARKDIRS]      = { set_bool_option, &shopt_markdirs, },
-    [SHOPT_EXTENDEDGLOB]  = { set_bool_option, &shopt_extendedglob, },
-    [SHOPT_NULLGLOB]      = { set_bool_option, &shopt_nullglob, },
-    [SHOPT_BRACEEXPAND]   = { set_bool_option, &shopt_braceexpand, },
-    [SHOPT_CURASYNC]      = { set_bool_option, &shopt_curasync, },
-    [SHOPT_ERREXIT]       = { set_bool_option, &shopt_errexit, },
-    [SHOPT_NOUNSET]       = { set_bool_option, &shopt_nounset, },
-    [SHOPT_NOEXEC]        = { set_bool_option, &shopt_noexec, },
-    [SHOPT_IGNOREEOF]     = { set_bool_option, &shopt_ignoreeof, },
-    [SHOPT_VERBOSE]       = { set_bool_option, &shopt_verbose, },
-    [SHOPT_XTRACE]        = { set_bool_option, &shopt_xtrace, },
-    [SHOPT_HISTSPACE]     = { set_bool_option, &shopt_histspace, },
-    [SHOPT_NOLOG]         = { set_ignore_option, NULL, },
-    [SHOPT_MONITOR]       = { set_monitor_option, NULL, },
-    [SHOPT_NOTIFY]        = { set_bool_option, &shopt_notify, },
+    [SHOPT_ALLEXPORT]      = { set_bool_option, &shopt_allexport, },
+    [SHOPT_HASHONDEF]      = { set_bool_option, &shopt_hashondef, },
+    [SHOPT_NOCLOBBER]      = { set_bool_option, &shopt_noclobber, },
+    [SHOPT_NOGLOB]         = { set_bool_option, &shopt_noglob, },
+    [SHOPT_NOCASEGLOB]     = { set_bool_option, &shopt_nocaseglob, },
+    [SHOPT_DOTGLOB]        = { set_bool_option, &shopt_dotglob, },
+    [SHOPT_MARKDIRS]       = { set_bool_option, &shopt_markdirs, },
+    [SHOPT_EXTENDEDGLOB]   = { set_bool_option, &shopt_extendedglob, },
+    [SHOPT_NULLGLOB]       = { set_bool_option, &shopt_nullglob, },
+    [SHOPT_BRACEEXPAND]    = { set_bool_option, &shopt_braceexpand, },
+    [SHOPT_CURASYNC]       = { set_bool_option, &shopt_curasync, },
+    [SHOPT_CURBG]          = { set_bool_option, &shopt_curbg, },
+    [SHOPT_CURSTOP]        = { set_bool_option, &shopt_curstop, },
+    [SHOPT_ERREXIT]        = { set_bool_option, &shopt_errexit, },
+    [SHOPT_NOUNSET]        = { set_bool_option, &shopt_nounset, },
+    [SHOPT_NOEXEC]         = { set_bool_option, &shopt_noexec, },
+    [SHOPT_IGNOREEOF]      = { set_bool_option, &shopt_ignoreeof, },
+    [SHOPT_VERBOSE]        = { set_bool_option, &shopt_verbose, },
+    [SHOPT_XTRACE]         = { set_bool_option, &shopt_xtrace, },
+    [SHOPT_HISTSPACE]      = { set_bool_option, &shopt_histspace, },
+    [SHOPT_NOLOG]          = { set_ignore_option, NULL, },
+    [SHOPT_MONITOR]        = { set_monitor_option, NULL, },
+    [SHOPT_NOTIFY]         = { set_bool_option, &shopt_notify, },
 #if YASH_ENABLE_LINEEDIT
-    [SHOPT_NOTIFYLE]      = { set_bool_option, &shopt_notifyle, },
+    [SHOPT_NOTIFYLE]       = { set_bool_option, &shopt_notifyle, },
 #endif
-    [SHOPT_POSIX]         = { set_bool_option, &posixly_correct, },
+    [SHOPT_POSIX]          = { set_bool_option, &posixly_correct, },
 #if YASH_ENABLE_LINEEDIT
-    [SHOPT_VI]            = { set_lineedit_option, NULL, },
-    [SHOPT_EMACS]         = { set_lineedit_option, NULL, },
-    [SHOPT_LE_CONVMETA]   = { set_le_convmeta_option, NULL, },
-    [SHOPT_LE_NOCONVMETA] = { set_le_noconvmeta_option, NULL, },
-    [SHOPT_LE_PROMPTSP]   = { set_bool_option, &shopt_le_promptsp, },
+    [SHOPT_VI]             = { set_lineedit_option, NULL, },
+    [SHOPT_EMACS]          = { set_lineedit_option, NULL, },
+    [SHOPT_LE_CONVMETA]    = { set_le_convmeta_option, NULL, },
+    [SHOPT_LE_NOCONVMETA]  = { set_le_noconvmeta_option, NULL, },
+    [SHOPT_LE_VISIBLEBELL] = { set_bool_option, &shopt_le_visiblebell, },
+    [SHOPT_LE_PROMPTSP]    = { set_bool_option, &shopt_le_promptsp, },
 #endif
+#if YASH_ENABLE_HELP
     //[SHOPT_HELP]
+#endif
     { 0, NULL, },
 };
 
+static const struct xoption all_help_options[] = {
+    { L"all",  xno_argument, L'a', },
+#if YASH_ENABLE_HELP
+    { L"help", xno_argument, L'-', }, /* must be next to "all" */
+#endif
+    { NULL, 0, 0, },
+};
+
 const struct xoption *const shell_long_options = long_options;
 const struct xoption *const set_long_options   = long_options + SHOPT_setopt;
-const struct xoption *const help_option        = long_options + SHOPT_HELP;
+const struct xoption *const all_option         = all_help_options;
+const struct xoption *const help_option        = all_help_options + 1;
 
 /* The index of the currently parsing option in the `long_options' and
  * `setoptinfo' arrays. */
@@ -266,7 +296,7 @@ void set_option(void)
     info->func(info->argp);
 }
 
-/* Sets the option specified by the character `c'.
+/* Sets the option specified by character `c'.
  * An unrecognized character is ignored. */
 void set_single_option(wchar_t c)
 {
@@ -287,8 +317,8 @@ void set_single_option(wchar_t c)
     set_option();
 }
 
-/* Switches the setting of a specified long option.
- * Options that can only be used in shell invocation are ignored.
+/* Switches the setting of the specified long option.
+ * Options that can only be used in shell initialization are ignored.
  * Returns true if successful, false for invalid options. */
 bool set_long_option(const wchar_t *s)
 {
@@ -348,9 +378,9 @@ void set_lineedit_option(void *argp __attribute__((unused)))
     }
     if (xoptopt == L'-') {
 	shopt_lineedit = opt;
-	/* turn on lineedit */
+	/* turn on line editing */
     } else {
-	/* turn off lineedit */
+	/* turn off line editing */
 	if (shopt_lineedit == opt)
 	    shopt_lineedit = shopt_nolineedit;
     }
@@ -362,9 +392,8 @@ void set_le_convmeta_option(void *argp __attribute__((unused)))
     bool value = (xoptopt == L'-');
     if (value)
 	shopt_le_convmeta = shopt_yes;
-    else
-	if (shopt_le_convmeta == shopt_yes)
-	    shopt_le_convmeta = shopt_auto;
+    else if (shopt_le_convmeta == shopt_yes)
+	shopt_le_convmeta = shopt_auto;
 }
 
 /* Changes the setting of the le-noconvmeta option. */
@@ -373,14 +402,14 @@ void set_le_noconvmeta_option(void *argp __attribute__((unused)))
     bool value = (xoptopt == L'-');
     if (value)
 	shopt_le_convmeta = shopt_no;
-    else
-	if (shopt_le_convmeta == shopt_no)
-	    shopt_le_convmeta = shopt_auto;
+    else if (shopt_le_convmeta == shopt_no)
+	shopt_le_convmeta = shopt_auto;
 }
 
 #endif /* YASH_ENABLE_LINEEDIT */
 
-/* Return current value of special parameter $- as a newly malloced string. */
+/* Returns the current value of special parameter $- as a newly malloced string.
+ */
 wchar_t *get_hyphen_parameter(void)
 {
     xwcsbuf_T buf;
@@ -411,6 +440,7 @@ wchar_t *get_hyphen_parameter(void)
 static int set_builtin_print_current_settings(void);
 static int set_builtin_print_restoring_commands(void);
 
+/* The "set" builtin. */
 int set_builtin(int argc, void **argv)
 {
     wchar_t opt;
@@ -444,8 +474,10 @@ int set_builtin(int argc, void **argv)
 		    goto optionerror;
 		}
 		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    case L'?':  optionerror:
 		fprintf(stderr,
 		    gt("Usage:  set [-abefhmnuvxC] [-o option] [+o option] "
@@ -479,6 +511,8 @@ int set_builtin_print_current_settings(void)
     PRINTSETTING(allexport, shopt_allexport);
     PRINTSETTING(braceexpand, shopt_braceexpand);
     PRINTSETTING(curasync, shopt_curasync);
+    PRINTSETTING(curbg, shopt_curbg);
+    PRINTSETTING(curstop, shopt_curstop);
     PRINTSETTING(dotglob, shopt_dotglob);
 #if YASH_ENABLE_LINEEDIT
     PRINTSETTING(emacs, shopt_lineedit == shopt_emacs);
@@ -493,6 +527,7 @@ int set_builtin_print_current_settings(void)
     PRINTSETTING(le-convmeta, shopt_le_convmeta == shopt_yes);
     PRINTSETTING(le-noconvmeta, shopt_le_convmeta == shopt_no);
     PRINTSETTING(le-promptsp, shopt_le_promptsp);
+    PRINTSETTING(le-visiblebell, shopt_le_visiblebell);
 #endif
     PRINTSETTING(login, is_login_shell);
     PRINTSETTING(markdirs, shopt_markdirs);
@@ -533,6 +568,8 @@ int set_builtin_print_restoring_commands(void)
     PRINTSETTING(allexport, shopt_allexport);
     PRINTSETTING(braceexpand, shopt_braceexpand);
     PRINTSETTING(curasync, shopt_curasync);
+    PRINTSETTING(curbg, shopt_curbg);
+    PRINTSETTING(curstop, shopt_curstop);
     PRINTSETTING(dotglob, shopt_dotglob);
 #if YASH_ENABLE_LINEEDIT
     PRINTSETTING(emacs, shopt_lineedit == shopt_emacs);
@@ -547,6 +584,7 @@ int set_builtin_print_restoring_commands(void)
     PRINTSETTING(le-convmeta, shopt_le_convmeta == shopt_yes);
     PRINTSETTING(le-noconvmeta, shopt_le_convmeta == shopt_no);
     PRINTSETTING(le-promptsp, shopt_le_promptsp);
+    PRINTSETTING(le-visiblebell, shopt_le_visiblebell);
 #endif
     //PRINTSETTING(login, is_login_shell);
     PRINTSETTING(markdirs, shopt_markdirs);
@@ -578,6 +616,7 @@ int set_builtin_print_restoring_commands(void)
     }
 }
 
+#if YASH_ENABLE_HELP
 const char set_help[] = Ngt(
 "set - set shell options and positional parameters\n"
 "\tset [-abefhmnuvxC] [+abefhmnuvxC] [-o option] [+o option] [--] [arg...]\n"
@@ -625,8 +664,8 @@ const char set_help[] = Ngt(
 " -m --monitor\n"
 "\tEnable job control. All jobs are run in their own process group.\n"
 "\tWhen the status of a job is changed, the status is reported before\n"
-"\tthe next prompt. This option is enabled by default for interactive\n"
-"\tshells.\n"
+"\tthe next prompt. This option is enabled by default for an interactive\n"
+"\tshell.\n"
 " -n --noexec\n"
 "\tCommands are parsed, but not executed.\n"
 "\tUseful for syntax checking of a shell script file.\n"
@@ -639,15 +678,18 @@ const char set_help[] = Ngt(
 "\tAfter each command line is expanded, the expanded line is printed\n"
 "\tto the standard error.\n"
 " -C --noclobber\n"
-"\tPrevent existent files from being overridden by the \">\"\n"
-"\tredirection.\n"
+"\tPrevent existent files from being overridden by the \">\" redirection.\n"
 " --ignoreeof\n"
 "\tDo not exit when an EOF is entered.\n"
-"\tThis option is effective in interactive shells only.\n"
+"\tThis option is effective in an interactive shell only.\n"
 " --braceexpand\n"
 "\tEnable brace expansion.\n"
-" --curasync\n"
-"\tWhen a new background job is invoked, it becomes the current job.\n"
+" --curasync, --curbg, --curstop\n"
+"\tA background job becomes the current job when\n"
+"\t   (curasync)  invoked as an asynchronous command\n"
+"\t   (curbg)     resumed by the \"bg\" command\n"
+"\t   (curstop)   stopped.\n"
+"\t(These options are enabled by default)\n"
 " --histspace\n"
 "\tDo not save lines that start with a space in the history.\n"
 " --posix\n"
@@ -660,6 +702,8 @@ const char set_help[] = Ngt(
 "\tTreat 8th bit of input as a meta-key flag (regardless of terminfo).\n"
 " --le-noconvmeta\n"
 "\tDo not treat 8th bit of input as a meta-key flag.\n"
+" --le-visiblebell\n"
+"\tAlert with a flash rather than a bell.\n"
 " --le-promptsp\n"
 "\tMove cursor to beginning of line each time when starting\n"
 "\tline-editing. (enabled by default)\n"
@@ -667,6 +711,7 @@ const char set_help[] = Ngt(
 "Long options in the form of `--xxx' are equivalent to `-o xxx'.\n"
 "Use `+o xxx' to turn off a long option. You cannot use `+-xxx' or `++xxx'.\n"
 );
+#endif /* YASH_ENABLE_HELP */
 
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
