@@ -24,6 +24,7 @@
 #define YASH_PARSER_H
 
 #include <stddef.h>
+#include "input.h"
 
 
 /********** Parse Tree Structures **********/
@@ -44,7 +45,8 @@ typedef struct pipeline_T {
     struct command_T  *pl_commands;  /* commands in this pipeline */
     _Bool              pl_neg, pl_cond;
 } pipeline_T;
-/* pl_neg:  whether the exit status of the pipeline is inverted.
+/* pl_neg:  indicates this pipeline is prefix by "!", in which case the exit
+ *          status of the pipeline is inverted.
  * pl_cond: true for "&&", false for "||". Ignored for the first pipeline in an
  *          and/or list. */
 
@@ -110,10 +112,10 @@ typedef struct command_T {
 #define c_funcbody c_content.funcdef.funcbody
 /* `c_words' and `c_forwords' are NULL-terminated arrays of pointers to
  * `wordunit_T', cast to `void *'.
- * If `c_forwords' is NULL, the for loop doesn't have a "in" clause.
+ * If `c_forwords' is NULL, the for loop doesn't have the "in" clause.
  * If `c_forwords[0]' is NULL, the "in" clause is empty. */
 
-/* condition and commands of if command */
+/* condition and commands of an if command */
 typedef struct ifcommand_T {
     struct ifcommand_T *next;
     struct and_or_T    *ic_condition;  /* condition */
@@ -121,7 +123,7 @@ typedef struct ifcommand_T {
 } ifcommand_T;
 /* For a "else" clause, `next' and `ic_condition' are NULL. */
 
-/* patterns and commands of case command */
+/* patterns and commands of a case command */
 typedef struct caseitem_T {
     struct caseitem_T *next;
     void             **ci_patterns;  /* patterns to do matching */
@@ -130,6 +132,15 @@ typedef struct caseitem_T {
 /* `ci_patterns' is a NULL-terminated array of pointers to `wordunit_T',
  * cast to `void *'. */
 
+/* embedded command */
+typedef struct embedcmd_T {
+    _Bool is_preparsed;
+    union {
+	wchar_t  *unparsed;
+	and_or_T *preparsed;
+    } value;
+} embedcmd_T;
+
 /* type of wordunit_T */
 typedef enum {
     WT_STRING,  /* string (including quotes) */
@@ -145,7 +156,7 @@ typedef struct wordunit_T {
     union {
 	wchar_t           *string;  /* string (including quotes) */
 	struct paramexp_T *param;   /* parameter expansion */
-	wchar_t           *cmdsub;  /* commands for command substitution */
+	embedcmd_T         cmdsub;  /* command substitution */
 	struct wordunit_T *arith;   /* expression for arithmetic expansion */
     } wu_value;
 } wordunit_T;
@@ -234,7 +245,7 @@ typedef struct assign_T {
 #define a_scalar a_value.scalar
 #define a_array  a_value.array
 /* `a_scalar' may be NULL to denote an empty string.
- * `a_array' is an array of pointers to wordunit_T. */
+ * `a_array' is an array of pointers to `wordunit_T'. */
 
 /* type of redirection */
 typedef enum {
@@ -264,7 +275,7 @@ typedef struct redir_T {
 	    wchar_t *hereend;  /* token indicating end of here-document */
 	    struct wordunit_T *herecontent;  /* contents of here-document */
 	} heredoc;
-	wchar_t *command;
+	embedcmd_T command;
     } rd_value;
 } redir_T;
 #define rd_filename    rd_value.filename
@@ -280,45 +291,39 @@ typedef struct redir_T {
  * argument being true. */
 
 
-/********** Interfaces to parse routines **********/
+/********** Interface to Parsing Routines **********/
 
-struct xwcsbuf_T;
 struct parsestate_T;
 
-/* Type of a input function: function that reads input and appends it to the
- * buffer.
- * Input is done line-wise; one line is read at a time.
- * The input string is terminated by newline L'\n' except when there is no
- * newline at the end of input.
- * Returns 0 if some string is read successfully,
- *         1 if SIGINT is caught in an interactive shell, or
- *         EOF if EOF is input or an error occurs.
- * If the return value is non-zero, the buffer is unchanged.
- * The input function may be called even after it returned a non-zero. */
-typedef int inputfunc_T(struct xwcsbuf_T *buf, void *inputinfo);
-
 /* object containing the info for parsing */
 typedef struct parseinfo_T {
     _Bool print_errmsg;   /* print error messages? */
     _Bool enable_verbose; /* echo input if `shopt_verbose' is true? */
+#if YASH_ENABLE_ALIAS
+    _Bool enable_alias;   /* perform alias substitution? */
+#endif
     const char *filename; /* the input filename, which may be NULL */
     unsigned long lineno; /* line number, which should be initialized to 1 */
     inputfunc_T *input;   /* input function */
     void *inputinfo;      /* pointer passed to the input function */
     _Bool intrinput;      /* input is interactive? */
-    int lastinputresult;  /* last return value of input function */
+    inputresult_T lastinputresult;  /* last return value of input function */
 } parseinfo_T;
 /* If `intrinput' is true, `input' is `input_readline' and `inputinfo' is a
  * pointer to a `struct input_readline_info' object.
  * Note that input may not be from a terminal even if `intrinput' is true. */
 
+typedef enum parseresult_T {
+    PR_OK, PR_EOF, PR_SYNTAX_ERROR, PR_INPUT_ERROR,
+} parseresult_T;
+
 extern struct parsestate_T *save_parse_state(void)
     __attribute__((malloc,warn_unused_result));
 extern void restore_parse_state(struct parsestate_T *state)
     __attribute__((nonnull));
 
 
-extern int read_and_parse(
+extern parseresult_T read_and_parse(
 	parseinfo_T *restrict info, and_or_T **restrict result)
     __attribute__((nonnull));
 
@@ -327,7 +332,7 @@ extern _Bool parse_string(
     __attribute__((nonnull));
 
 
-/********** Auxiliary function **********/
+/********** Auxiliary Functions **********/
 
 extern _Bool is_name_char(wchar_t c)
     __attribute__((pure));
@@ -339,15 +344,15 @@ extern _Bool is_token_delimiter_char(wchar_t c)
     __attribute__((pure));
 
 
-/********** Functions that Convert Parse Trees into Strings **********/
+/********** Functions That Convert Parse Trees into Strings **********/
 
 extern wchar_t *pipelines_to_wcs(const pipeline_T *pipelines)
     __attribute__((malloc,warn_unused_result));
-extern wchar_t *command_to_wcs(const command_T *command)
+extern wchar_t *command_to_wcs(const command_T *command, _Bool multiline)
     __attribute__((malloc,warn_unused_result));
 
 
-/********** Functions that Free/Duplicate Parse Trees **********/
+/********** Functions That Free/Duplicate Parse Trees **********/
 
 extern void andorsfree(and_or_T *a);
 extern void wordfree(wordunit_T *w);
