@@ -61,19 +61,21 @@ static void print_version(void);
 static bool input_is_interactive_terminal(const parseinfo_T *pinfo)
     __attribute__((nonnull));
 
-/* the process ID of the shell */
+/* The process ID of the shell.
+ * This value does not change in a subshell. */
 pid_t shell_pid;
-/* the process group ID of the shell */
+/* The process group ID of the shell.
+ * In a job-controlled subshell, this value is set to the subshell's pgid. */
 pid_t shell_pgid;
-/* In a job-controlled subshell, `shell_pgid' is set to the subshell's pgid.
- * `shell_pid' is always the process ID of the main shell process. */
 
-/* an equivalent to the -f flag for "exit" builtin */
+/* If this flag is true when the "exit" builtin is invoked, the -f option is
+ * assumed to be specified. */
 static bool forceexit;
-/* the next value of `forceexit' */
+/* The next value of `forceexit'. */
 bool nextforceexit;
 
 
+/* The "main" function. The execution of the shell starts here. */
 int main(int argc, char **argv)
 {
     void *wargv[argc + 1];
@@ -107,6 +109,7 @@ int main(int argc, char **argv)
     }
     wargv[argc] = NULL;
 
+    /* parse argv[0] */
     yash_program_invocation_name = wargv[0] ? wargv[0] : L"";
     yash_program_invocation_short_name
 	= wcsrchr(yash_program_invocation_name, L'/');
@@ -225,12 +228,11 @@ int main(int argc, char **argv)
 #ifdef NDEBUG
     wchar_t *command;
     FILE *input;
-    const char *inputname;
 #else
     wchar_t *command = command;
     FILE *input = input;
-    const char *inputname = inputname;
 #endif
+    const char *inputname;
 
     if (shopt_read_arg && shopt_read_stdin) {
 	xerror(0, Ngt("both of -c and -s options cannot be used at once"));
@@ -238,12 +240,16 @@ int main(int argc, char **argv)
     }
     if (shopt_read_arg) {
 	command = wargv[xoptind++];
-	if (!command) {
+	if (command == NULL) {
 	    xerror(0, Ngt("-c option specified but no command given"));
 	    exit(Exit_ERROR);
 	}
-	if (xoptind < argc)
+	if (xoptind < argc) {
+	    inputname = argv[xoptind];
 	    command_name = wargv[xoptind++];
+	} else {
+	    inputname = posixly_correct ? "sh -c" : "yash -c";
+	}
     } else {
 	if (argc == xoptind)
 	    shopt_read_stdin = true;
@@ -258,7 +264,7 @@ int main(int argc, char **argv)
 	    input = fopen(inputname, "r");
 	    input = reopen_with_shellfd(input, "r", true);
 	    command_name = wargv[xoptind++];
-	    if (!input) {
+	    if (input == NULL) {
 		int errno_ = errno;
 		xerror(errno_, Ngt("cannot open `%s'"), inputname);
 		exit(errno_ == ENOENT ? Exit_NOTFOUND : Exit_NOEXEC);
@@ -283,27 +289,25 @@ int main(int argc, char **argv)
     }
     set_signals();
     set_positional_parameters(wargv + xoptind);
-    if (getuid() == geteuid() && getgid() == getegid()) {
-	if (!noprofile)
+
+    if (is_login_shell && !posixly_correct && !noprofile)
+	if (getuid() == geteuid() && getgid() == getegid())
 	    execute_profile();
-	if (!norcfile)
+    if (is_interactive && !norcfile)
+	if (getuid() == geteuid() && getgid() == getegid())
 	    execute_rcfile(rcfile);
-    }
 
     if (shopt_read_arg) {
-	exec_wcs(command, posixly_correct ? "sh -c" : "yash -c", true);
+	exec_wcs(command, inputname, true);
     } else {
-	exec_input(input, inputname, is_interactive, true);
+	exec_input(input, inputname, is_interactive, true, true);
     }
     assert(false);
 }
 
-/* Executes "$HOME/.yash_profile" if this is a login shell. */
+/* Executes "$HOME/.yash_profile". */
 static void execute_profile(void)
 {
-    if (!is_login_shell || posixly_correct)
-	return;
-
     wchar_t *wpath = parse_and_expand_string(
 	    L"$HOME/.yash_profile", NULL, false);
     if (wpath) {
@@ -311,7 +315,7 @@ static void execute_profile(void)
 	if (path) {
 	    FILE *f = reopen_with_shellfd(fopen(path, "r"), "r", true);
 	    if (f) {
-		exec_input(f, path, false, false);
+		exec_input(f, path, false, true, false);
 		fclose(f);
 	    }
 	    free(path);
@@ -319,7 +323,7 @@ static void execute_profile(void)
     }
 }
 
-/* Executes the initialization file if this is an interactive shell.
+/* Executes the initialization file.
  * `rcfile' is the filename to source.
  * If `rcfile' is NULL, it defaults to "$HOME/.yashrc" or "$ENV". */
 static void execute_rcfile(const wchar_t *rcfile)
@@ -327,11 +331,10 @@ static void execute_rcfile(const wchar_t *rcfile)
     wchar_t *wpath;
     char *path;
     FILE *f;
-    if (!is_interactive)
-	return;
+
     if (posixly_correct) {
 	const wchar_t *env = getvar(L VAR_ENV);
-	if (!env)
+	if (env == NULL)
 	    return;
 	wpath = parse_and_expand_string(env, "$ENV", false);
 	if (!wpath)
@@ -347,41 +350,44 @@ static void execute_rcfile(const wchar_t *rcfile)
 	    path = realloc_wcstombs(wpath);
 	}
     }
-    if (!path)
+    if (path == NULL)
 	return;
     f = reopen_with_shellfd(fopen(path, "r"), "r", true);
-    if (f) {
-	exec_input(f, path, false, false);
+    if (f != NULL) {
+	exec_input(f, path, false, true, false);
 	fclose(f);
     }
     free(path);
 }
 
 /* Exits the shell with the specified exit status.
- * If `status' is negative, the value of `laststatus' is used.
- * This function executes EXIT trap and calls `reset_own_pgid'.
+ * When this function is first called and `status' is negative, the value of
+ * `laststatus' is used for `status'. When this function is called again and
+ * `status' is negative, the value for the first call is used.
+ * This function executes the EXIT trap.
  * This function never returns.
  * This function is reentrant and exits immediately if reentered. */
 void exit_shell_with_status(int status)
 {
-    static bool exiting = false;
-    int exitstatus;
+    static int exitstatus = -1;
 
     if (status >= 0)
 	laststatus = status;
-    exitstatus = laststatus;
-    if (!exiting) {
-	exiting = true;
+    assert(laststatus >= 0);
+    if (exitstatus < 0) {
+	exitstatus = laststatus;
 	execute_exit_trap();
-#if YASH_ENABLE_HISTORY
-	if (is_interactive_now)
-	    finalize_history();
-#endif
+    } else {
+	if (status >= 0)
+	    exitstatus = status;
     }
+#if YASH_ENABLE_HISTORY
+    finalize_history();
+#endif
     exit(exitstatus);
 }
 
-/* Prints the help message to stdout */
+/* Prints the help message to the standard output. */
 void print_help(void)
 {
     if (posixly_correct) {
@@ -404,7 +410,7 @@ void print_help(void)
     }
 }
 
-/* Prints the version info to stdout. */
+/* Prints the version info to the standard output. */
 void print_version(void)
 {
     printf(gt("Yet another shell, version %s\n"), PACKAGE_VERSION);
@@ -450,36 +456,11 @@ void print_version(void)
 }
 
 
-/********** Functions to execute commands **********/
+/********** Functions to Execute Commands **********/
 
-/* Parses a multibyte string and executes the commands.
- * `code' must start in a initial shift state.
+/* Parses the specified wide string and executes it as commands.
  * `name' is printed in an error message on syntax error. `name' may be NULL.
- * If there are no commands in `code', `laststatus' is set to 0. */
-void exec_mbs(const char *code, const char *name, bool finally_exit)
-{
-    struct input_mbs_info iinfo = {
-	.src = code,
-	.srclen = strlen(code) + 1,
-    };
-    struct parseinfo_T pinfo = {
-	.print_errmsg = true,
-	.enable_verbose = false,
-	.filename = name,
-	.lineno = 1,
-	.input = input_mbs,
-	.inputinfo = &iinfo,
-	.intrinput = false,
-	.lastinputresult = 0,
-    };
-    memset(&iinfo.state, 0, sizeof iinfo.state);  // initialize the shift state
-
-    parse_and_exec(&pinfo, finally_exit);
-}
-
-/* Parses a wide string and executes the commands.
- * `name' is printed in an error message on syntax error. `name' may be NULL.
- * If there are no commands in `code', `laststatus' is set to 0. */
+ * If there are no commands in `code', `laststatus' is set to zero. */
 void exec_wcs(const wchar_t *code, const char *name, bool finally_exit)
 {
     struct input_wcs_info iinfo = {
@@ -488,32 +469,37 @@ void exec_wcs(const wchar_t *code, const char *name, bool finally_exit)
     struct parseinfo_T pinfo = {
 	.print_errmsg = true,
 	.enable_verbose = false,
+#if YASH_ENABLE_ALIAS
+	.enable_alias = true,
+#endif
 	.filename = name,
 	.lineno = 1,
 	.input = input_wcs,
 	.inputinfo = &iinfo,
 	.intrinput = false,
-	.lastinputresult = 0,
     };
 
     parse_and_exec(&pinfo, finally_exit);
 }
 
-/* Parses input from a file stream and executes the commands.
- * The stream `f' must be set to non-blocking if it is not stdin.
+/* Parses the input from the specified file stream and executes commands.
+ * Stream `f' must be set to non-blocking if it is not `stdin'.
  * `name' is printed in an error message on syntax error. `name' may be NULL.
  * If `intrinput' is true, the input stream is considered interactive.
- * If there are no commands in input, `laststatus' is set to 0. */
-void exec_input(FILE *f, const char *name, bool intrinput, bool finally_exit)
+ * If there are no commands in the input, `laststatus' is set to zero. */
+void exec_input(FILE *f, const char *name,
+	bool intrinput, bool enable_alias, bool finally_exit)
 {
     struct input_readline_info rlinfo;
     struct parseinfo_T pinfo = {
 	.print_errmsg = true,
 	.enable_verbose = true,
+#if YASH_ENABLE_ALIAS
+	.enable_alias = enable_alias,
+#endif
 	.filename = name,
 	.lineno = 1,
 	.intrinput = intrinput,
-	.lastinputresult = 0,
     };
     if (intrinput) {
 	rlinfo.fp = f;
@@ -525,10 +511,14 @@ void exec_input(FILE *f, const char *name, bool intrinput, bool finally_exit)
 	pinfo.inputinfo = f;
     }
     parse_and_exec(&pinfo, finally_exit);
+
+#if !YASH_ENABLE_ALIAS
+    (void) enable_alias;  // suppress compiler warning
+#endif
 }
 
-/* Parses input using the specified `parseinfo_T' and executes the commands.
- * If there are no commands in `code', `laststatus' is set to 0. */
+/* Parses the input using the specified `parseinfo_T' and executes commands.
+ * If no commands were executed, `laststatus' is set to zero. */
 void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 {
 #ifdef NDEBUG
@@ -542,8 +532,6 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
     bool executed = false;
 
     for (;;) {
-	and_or_T *commands;
-
 	if (pinfo->intrinput) {
 	    forceexit = nextforceexit;
 	    nextforceexit = false;
@@ -554,31 +542,24 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 	    reset_sigint();
 	    reset_execinfo();
 	} else {
-	    if (return_pending()) {
-		if (finally_exit) {
-		    wchar_t argv0[] = L"return";
-		    reset_execinfo();
-		    exit_builtin(1, (void *[]) { argv0, NULL });
-		    continue;
-		} else {
-		    goto out;
-		}
-	    }
+	    if (return_pending())
+		goto out;
 	}
 
+	and_or_T *commands;
 	switch (read_and_parse(pinfo, &commands)) {
-	    case 0:  // OK
+	    case PR_OK:
 		if (commands) {
 		    if (!shopt_noexec) {
 			exec_and_or_lists(commands,
 				finally_exit && !pinfo->intrinput &&
-				pinfo->lastinputresult == EOF);
+				pinfo->lastinputresult == INPUT_EOF);
 			executed = true;
 		    }
 		    andorsfree(commands);
 		}
 		break;
-	    case EOF:
+	    case PR_EOF:
 		if (shopt_ignoreeof && input_is_interactive_terminal(pinfo)) {
 		    fprintf(stderr, gt("Use `exit' to leave the shell.\n"));
 		    break;
@@ -591,21 +572,22 @@ void parse_and_exec(parseinfo_T *pinfo, bool finally_exit)
 		} else
 		    goto out;
 		break;
-	    case 1:  // syntax error
-		laststatus = Exit_SYNERROR;
-		if (pinfo->intrinput)
-		    break;
-		else if (finally_exit)
-		    exit_shell();
-		else
+	    case PR_SYNTAX_ERROR:
+		if (!is_interactive_now)
+		    exit_shell_with_status(Exit_SYNERROR);
+		if (!pinfo->intrinput) {
+		    laststatus = Exit_SYNERROR;
 		    goto out;
+		}
+		break;
+	    case PR_INPUT_ERROR:
+		laststatus = Exit_ERROR;
+		goto out;
 	}
     }
-    /* If no commands are executed, set `laststatus' to 0 finally.
-     * We don't set it at first because the value of "$?" gets wrong in
-     * execution. */
 out:
-    assert(!finally_exit);
+    if (finally_exit)
+	exit_shell();
     load_execinfo(ei);
 }
 
@@ -624,11 +606,13 @@ bool input_is_interactive_terminal(const parseinfo_T *pinfo)
 
 static const struct xoption force_help_options[] = {
     { L"force", xno_argument, L'f', },
+#if YASH_ENABLE_HELP
     { L"help",  xno_argument, L'-', },
+#endif
     { NULL, 0, 0, },
 };
 
-/* "exit" builtin.
+/* The "exit" builtin.
  * If the shell is interactive, there are stopped jobs and the -f flag is not
  * specified, then prints a warning message and does not exit. */
 int exit_builtin(int argc, void **argv)
@@ -641,8 +625,10 @@ int exit_builtin(int argc, void **argv)
 	    case L'f':
 		forceexit = true;
 		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr, gt("Usage:  exit [-f] [n]\n"));
 		SPECIAL_BI_ERROR;
@@ -668,9 +654,7 @@ int exit_builtin(int argc, void **argv)
     int status;
     const wchar_t *statusstr = ARGV(xoptind);
     if (statusstr != NULL) {
-	if (xwcstoi(statusstr, 10, &status) && status >= 0) {
-	    status &= 0xFF;
-	} else {
+	if (!xwcstoi(statusstr, 10, &status) || status < 0) {
 	    xerror(0, Ngt("`%ls' is not a valid integer"), statusstr);
 	    status = Exit_ERROR;
 	    SPECIAL_BI_ERROR;
@@ -682,6 +666,7 @@ int exit_builtin(int argc, void **argv)
     assert(false);
 }
 
+#if YASH_ENABLE_HELP
 const char exit_help[] = Ngt(
 "exit - exit shell\n"
 "\texit [-f] [n]\n"
@@ -692,6 +677,7 @@ const char exit_help[] = Ngt(
 "a warning message and does not exit. Use the -f (--force) option or use\n"
 "`exit' twice in a row to avoid the warning and really exit.\n"
 );
+#endif
 
 /* The "suspend" builtin, which accepts the following options:
  *  -f: suspend even if it may cause a deadlock. */
@@ -706,8 +692,10 @@ int suspend_builtin(int argc, void **argv)
 	    case L'f':
 		force = true;
 		break;
+#if YASH_ENABLE_HELP
 	    case L'-':
 		return print_builtin_help(ARGV(0));
+#endif
 	    default:  print_usage:
 		fprintf(stderr, gt("Usage:  suspend [-f]\n"));
 		return Exit_ERROR;
@@ -721,9 +709,7 @@ int suspend_builtin(int argc, void **argv)
 	return Exit_FAILURE;
     }
 
-    bool ok;
-
-    ok = send_sigstop_to_myself();
+    bool ok = send_sigstop_to_myself();
     if (!ok)
 	xerror(errno, Ngt("cannot send SIGSTOP signal"));
     if (doing_job_control_now)
@@ -731,13 +717,16 @@ int suspend_builtin(int argc, void **argv)
     return ok ? Exit_SUCCESS : Exit_FAILURE;
 }
 
+#if YASH_ENABLE_HELP
 const char suspend_help[] = Ngt(
 "suspend - suspend shell\n"
 "\tsuspend [-f]\n"
 "Suspends the shell until it receives SIGCONT.\n"
-"This command refuses to suspend the shell if it is interactive and is a\n"
-"session leader.\n"
+"If the shell is interactive and is a session leader, this command refuses to\n"
+"suspend it in order to avoid a possible deadlock. You can use the -f option\n"
+"to force the shell to suspend anyway.\n"
 );
+#endif
 
 
 /* vim: set ts=8 sts=4 sw=4 noet tw=80: */
